/*
	File Name:	ae.c
*/

#include <EPPC.h>
#include <AppleEvents.h>

#include "globs.h"
#include "ae.h"

struct triplets {
	AEEventClass		theEventClass;
	AEEventID			theEventID;
	AEEventHandlerUPP	theHandler;
};
typedef struct triplets triplets;

static triplets keywordsToInstall[] = {
	{ kCoreEventClass,		kAEOpenApplication,		(AEEventHandlerUPP)DoAEOpenApplication },
	{ kCoreEventClass,		kAEOpenDocuments,		(AEEventHandlerUPP)DoAEOpenDocuments },
	{ kCoreEventClass,		kAEPrintDocuments,		(AEEventHandlerUPP)DoAEPrintDocuments },
	{ kCoreEventClass,		kAEQuitApplication,		(AEEventHandlerUPP)DoAEQuitApplication }
		/* The above are the four required AppleEvents. */
};

/********************************************************************

	InitAE
	
********************************************************************/

OSErr InitAE(void)
{
	OSErr	err = noErr;
	short	i;

	for (i = 0; i < (sizeof(keywordsToInstall) / sizeof(triplets)); ++i) {
		err = AEInstallEventHandler(keywordsToInstall[i].theEventClass,
									keywordsToInstall[i].theEventID,
									NewAEEventHandlerProc(keywordsToInstall[i].theHandler),
									0L,		/* unused refcon */
									false);	/* not system handler */
		if (err != noErr) return err;
	}
	return err;
}

/********************************************************************

	DoAEOpenApplication
	
	Do things here if we are opened with no documents, open new document
	for example.
	
********************************************************************/

pascal OSErr DoAEOpenApplication(AppleEvent *message, AppleEvent *reply, long refcon)
{
	OSErr err = noErr;
	
	return err;
}

/********************************************************************

	DoAEOpenDocuments
	
********************************************************************/

pascal OSErr DoAEOpenDocuments(AppleEvent *message, AppleEvent *reply, long refcon)
{
	OSErr		err = noErr;
	AEDesc		fileListDesc;
	long		numFiles;
	DescType	actualType;
	long		actualSize;
	AEKeyword	actualKeyword;
	FSSpec		oneFile;
	long		index;
							
	/* extract the list of aliases into fileListDesc */
	err = AEGetKeyDesc(message, keyDirectObject, typeAEList, &fileListDesc);
	if (err != noErr) return err;
		
	/* count the list elements */
	err = AECountItems(&fileListDesc, &numFiles);
	if (err != noErr) { AEDisposeDesc(&fileListDesc); return err; }
	
	/* get each from list and process it */		
	for (index = 1; index <= numFiles; index ++) {
		err = AEGetNthPtr(&fileListDesc, index, typeFSS, &actualKeyword,
							&actualType, (Ptr)&oneFile, sizeof(oneFile), &actualSize);
		if (err != noErr) {
			AEDisposeDesc(&fileListDesc);
			return err;
		}
		
		/* oneFile contains FSSpec of file in question */
		SysBeep(0);
	}
	
	AEDisposeDesc(&fileListDesc);
	return err;
}

/********************************************************************

	DoAEPrintDocuments
	
********************************************************************/

pascal OSErr DoAEPrintDocuments(AppleEvent *message, AppleEvent *reply, long refcon)
{
	OSErr		err = noErr;
	AEDesc		fileListDesc;
	long		numFiles;
	DescType	actualType;
	long		actualSize;
	AEKeyword	actualKeyword;
	FSSpec		oneFile;
	long		index;
							
	/* extract the list of aliases into fileListDesc */
	err = AEGetKeyDesc(message, keyDirectObject, typeAEList, &fileListDesc);
	if (err != noErr) return err;
		
	/* count the list elements */
	err = AECountItems(&fileListDesc, &numFiles);
	if (err != noErr) { AEDisposeDesc(&fileListDesc); return err; }
	
	/* get each from list and process it */		
	for (index = 1; index <= numFiles; index ++) {
		err = AEGetNthPtr(&fileListDesc, index, typeFSS, &actualKeyword,
							&actualType, (Ptr)&oneFile, sizeof(oneFile), &actualSize);
		if (err != noErr) {
			AEDisposeDesc(&fileListDesc);
			return err;
		}
		
		/* oneFile contains FSSpec of file in question */
		SysBeep(0);
	}
	AEDisposeDesc(&fileListDesc);
	return err;
}

/********************************************************************

	DoAEQuitApplication
	
********************************************************************/

pascal OSErr DoAEQuitApplication(AppleEvent *message, AppleEvent *reply, long refcon)
{
	OSErr	err = noErr;

	// do our stuff here

	gDone = true;
	return err;
}