/*
	File Name:	main.c
*/

#include <EPPC.h>
#include <AppleEvents.h>

#include "globs.h"
#include "init.h"
#include "utils.h"
#include "prefs.h"
#include "modules.h"
#include "main.h"

/********************************************************************

	main
	
********************************************************************/

void main(void)
{
	/* initialize ourselves */
	InitMacintosh();
	InitGlobals();
	UnloadSeg(&InitMacintosh);

	/* handle events */
	HandleEvent();
	
	/* get out */
	CleanUp();
}

/********************************************************************

	CleanUp
	
********************************************************************/

void CleanUp(void)
{
	ExitToShell();
}

/********************************************************************

	HandleEvent
	
********************************************************************/

void HandleEvent(void)
{	
	while (!gDone) {
		
		Boolean gotEvent = WaitNextEvent(everyEvent, &gTheEvent, gSleepTime, nil);
		
		if (gotEvent) {
			
			switch(gTheEvent.what) {
			
				case nullEvent:
					IdleTime();
					break;
					
				case mouseDown:
					DoMouseDown();
					break;
					
				case updateEvt:
					/*
					{
						GrafPtr	savePort;
						GetPort(&savePort);
						SetPort((WindowPtr)gTheEvent.message);
						BeginUpdate((WindowPtr)gTheEvent.message);
						
						EndUpdate((WindowPtr)gTheEvent.message);
						SetPort(savePort);
					}
					*/
					break;
						
				case autoKey:
				case keyDown:
					if (gTheEvent.modifiers & cmdKey) {
						AdjustMenus();
						DoMenuCommand(MenuKey(gTheEvent.message & charCodeMask));
					}
					break;
					
				case app4Evt:
					gInBackground = (!(gTheEvent.message & 0x0001));
					InitCursor();
					break;
					
				case kHighLevelEvent: 
					{
						OSErr err = AEProcessAppleEvent(&gTheEvent);
						if (err != noErr) {
							NumToString((long)err, gStr);
							ParamText(gStr, "\p", "\p", "\p");
							StopAlert(alertAEProcessAEError, nil);
							ParamText("\p", "\p", "\p", "\p");
						}
					}
					break;
					
				default:
					break;
			}
		}
	}
}

/********************************************************************

	DoMouseDown
	
********************************************************************/

void DoMouseDown(void)
{
	WindowPtr	window;
	short		thePart = FindWindow(gTheEvent.where, &window);
		
	switch(thePart) {
	
		case inMenuBar:
			AdjustMenus();
			DoMenuCommand(MenuSelect(gTheEvent.where));
			break;
			
		case inSysWindow:
			SystemClick(&gTheEvent, window);
			break;
			
		case inGrow:
			{
				long newSize = GrowWindow(window, gTheEvent.where, &qd.screenBits.bounds);
				if (newSize)
					SizeWindow(window, LoWord(newSize), HiWord(newSize), true);
			}
			break;
			
		case inContent:
			if (window != FrontWindow())
				SelectWindow(window);
			break;
			
		case inDrag:
			DragWindow(window, gTheEvent.where, &qd.screenBits.bounds);
			break;
			
		case inGoAway:
			if (TrackGoAway(window, gTheEvent.where))
				CloseWindow(window);
			break;
			
		case inZoomIn:
		case inZoomOut:
			if (TrackBox(window, gTheEvent.where, thePart)) {
				SetPort(window);
				EraseRect(&window->portRect);
				ZoomWindow(window, thePart, true);
				InvalRect(&window->portRect);
			}
			break;
	}
}

/********************************************************************

	DoMenuCommand
	
********************************************************************/

void DoMenuCommand(long menuResult)
{
	short		menuID;
	short		menuItem;
	short		daRefNum;
	WindowPtr	window;
	OSErr		err = noErr;
	
	window = FrontWindow();
	menuID = HiWord(menuResult); 
	menuItem = LoWord(menuResult);
	
	switch(menuID) {
		
		case menuApple:
			switch(menuItem) {
				case itemAbout:
					Alert(alertAbout, nil);
					break;
					
				default:
					GetItem(GetMHandle(menuApple), menuItem, gStr);
					daRefNum = OpenDeskAcc(gStr);
					break;
			}
			break;
		
		case menuFile:
			switch(menuItem) {
				case itemQuit:
					gDone = true;
					break;
				default:
					SysBeep(0);
					break;
			}
			break;
			
		case menuEdit:
			switch(menuItem) {
				case itemSelectAll:
					break;
				
				case itemPreferences:
					DoPreferences();
					break;
				
				default:	
					SystemEdit(menuItem-1);
			}
			break;
			
		case menuCode:
			switch(menuItem) {
				case itemExecuteResource:
					err = ExecuteResource();
					if (err != noErr) {
						NumToString((long)err, gStr);
						ParamText(gStr, "\p", "\p", "\p");
						StopAlert(alertExecuteResourceError, nil);
						ParamText("\p", "\p", "\p", "\p");
					}
					break;
			#ifdef __powerc
				case itemExecuteSharedLib: {
					Str255	errName;
					err = ExecuteSharedLib(errName);
					if (err != noErr) {
						NumToString((long)err, gStr);
						ParamText(gStr, errName, "\p", "\p");
						StopAlert(alertExecuteSharedLibError, nil);
						ParamText("\p", "\p", "\p", "\p");
					}
					break;
				}
			#endif
			}
			break;
			
		default:
			break;
	}
	
	HiliteMenu(0);
} 

/********************************************************************

	AdjustMenus
	
********************************************************************/

void AdjustMenus(void)
{
	WindowPtr	frontmost;
	MenuHandle	menu;

	frontmost = FrontWindow();
	
	menu = GetMHandle(menuApple);
	EnableItem(menu, 0);
	menu = GetMHandle(menuFile);
	EnableItem(menu, 0);
	menu = GetMHandle(menuEdit);
	EnableItem(menu, 0);
	DisableItem(menu, itemUndo);
	DisableItem(menu, itemCut);
	DisableItem(menu, itemCopy);
	DisableItem(menu, itemPaste);
	DisableItem(menu, itemClear);
	DisableItem(menu, itemSelectAll);
	
	menu = GetMHandle(menuCode);
	EnableItem(menu, 0);
	#ifndef __powerc
	DisableItem(menu, itemExecuteSharedLib);
	#endif
	
	
/*
	if (IsDAWindow(frontmost)) {
		menu = GetMHandle(menuEdit);
		EnableItem(menu, 0);
	} else {
		menu = GetMHandle(menuEdit);
		DisableItem(menu, 0);
	}
*/
	
	DrawMenuBar();
} 

/********************************************************************

	IdleTime
	
********************************************************************/

void IdleTime(void)
{	
	if (gInIdle == true)
		return;
	
	gInIdle = true;
	
	/* do stuff here */
	
	gInIdle = false;
}

