/*
	File Name:	INIT.c
*/


#ifdef USESROUTINEDESCRIPTORS
#undef USESROUTINEDESCRIPTORS
#define USESROUTINEDESCRIPTORS 1
#else
#define USESROUTINEDESCRIPTORS 1
#endif

/*** includes ***********************************************************/

#include <LowMem.h>
#include <ToolUtils.h>
#include <OSUtils.h>
#include <Dialogs.h>
#include <Files.h>
#include <Menus.h>
#include <Script.h>
#include <Resources.h>
#include <Types.h>
#include <Memory.h>
#include <GestaltEqu.h>
#include <Errors.h>
#include <Folders.h>
#include <FragLoad.h>
#include <MixedMode.h>
#include <ConditionalMacros.h>

#include "INIT.h"
#include "common.h"
#include "A4Stuff.h"
#include "SetupA4.h"

/*** definitions ********************************************************/

#define kMinimumSystemVersion	0x00000700	// minimum system version

#define kINITResourceType		'INIT'		// our INIT code resource type
#define kINITResourceID			0			// and ID

#define kPPCResourceType		'PPC '		// our PowerPC code resource type
#define kPPCResourceID			0			// and ID

#define rStringListID				128			// INIT string list resource
#define iMenuScriptsFolderName		1			// MenuScripts folder name
#define iAnyApplicationFolderName	2

/*** typedefs ***********************************************************/

typedef pascal long (*MenuSelectProc)(Point pt);
typedef pascal long (*MenuKeyProc)(short ch);
typedef pascal void (*SystemMenuProc)(long menuResult);

/*** global variables ***************************************************/

GlobalsPtr		gGlobalsPtr;

/*** function prototypes ************************************************/

OSErr 	RememberMe(FSSpecPtr fsp);
pascal 	long MenuKey68k(short ch);
pascal 	long MenuSelect68k(Point startPt);
pascal 	void SystemMenu68k(long menuResult);
void	ApplyPatches(void * menuSelectCode, void * menuKeyCode, void * systemMenuCode);
OSErr	PatchPPC(void);

/*************************************************************************

	main

*************************************************************************/

void main(void)
{
	FSSpec	menuScriptsFolderFSSpec;
	FSSpec	anyApplicationFolderFSSpec;
	Str32	menuScriptsFolderName;
	Str32	anyApplicationFolderName;
	Handle	hINIT = nil;
	THz		savedZone;
	long	oldA4;
	long	response;
	long	foundDirID;
	short	foundVRefNum;
	OSErr	err = noErr;
	
	//
	// make sure we are in the system heap
	//
	
	savedZone = GetZone();
	SetZone(SystemZone());
	
	//
	// set up our A4 context for this file
	//
	
	oldA4 = SetCurrentA4();
	RememberA4();

	//
	// verify minimum system version
	//
	
	err = Gestalt(gestaltSystemVersion, &response);
	if ((err != noErr) || (response < kMinimumSystemVersion)) goto fail;

	//
	// allocate our global data
	//
	
	gGlobalsPtr = (GlobalsPtr)NewPtrSysClear(sizeof(Globals));
	if (gGlobalsPtr == nil) { 
		err = MemError() ? MemError() : memFullErr; 
		goto fail; 
	}
	
	//
	// find ourselves (its a Zen thing)
	//
	
	err = RememberMe(&gGlobalsPtr->ourFSSpec);
	if (err != noErr) goto fail;
	
	//
	// find out where our MenuScripts folder is located (in the System folder)
	// and figure out its directory id
	//
	
	// load folder name string from resource string list
	GetIndString(menuScriptsFolderName, rStringListID, iMenuScriptsFolderName);
	if (menuScriptsFolderName[0] == 0) {
		err = resNotFound;
		goto fail;
	}
	
	// locate dirID of system folder
	err = FindFolder(kOnSystemDisk, kSystemFolderType, kDontCreateFolder, &foundVRefNum, &foundDirID);
	if (err != noErr) goto fail;
	
	// make a spec for the MenuScripts folder
	err = FSMakeFSSpec(foundVRefNum, foundDirID, menuScriptsFolderName, &menuScriptsFolderFSSpec);
	if (err == fnfErr) {
		// create it if it doesnt exist and get its directory id
		err = FSpDirCreate(&menuScriptsFolderFSSpec, smSystemScript, &gGlobalsPtr->menuScriptsFolderDirID);
	} else if (err == noErr) {
		// otherwise just get its directory id
		err = GetDirectoryID(&menuScriptsFolderFSSpec, &gGlobalsPtr->menuScriptsFolderDirID);
	}
	if (err != noErr) goto fail;
		
	// load folder name string from resource string list
	GetIndString(anyApplicationFolderName, rStringListID, iAnyApplicationFolderName);
	if (anyApplicationFolderName[0] == 0) {
		err = resNotFound;
		goto fail;
	}
	
	// make a spec for the Any Application folder
	err = FSMakeFSSpec(foundVRefNum, gGlobalsPtr->menuScriptsFolderDirID, anyApplicationFolderName, &anyApplicationFolderFSSpec);
	if (err == fnfErr) {
		// create it if it doesnt exist and get its directory id
		err = FSpDirCreate(&anyApplicationFolderFSSpec, smSystemScript, &gGlobalsPtr->anyApplicationFolderDirID);
	} else if (err == noErr) {
		// otherwise just get its directory id
		err = GetDirectoryID(&anyApplicationFolderFSSpec, &gGlobalsPtr->anyApplicationFolderDirID);
	}
	if (err != noErr) goto fail;
	
	//
	// get a handle to our INIT resource
	//
	
	hINIT = Get1Resource(kINITResourceType, kINITResourceID);
	if (hINIT == nil) {
		err = ResError() ? ResError() : resNotFound;
		goto fail;
	}
	
	//
	// patch traps
	//
	
	err = Gestalt(gestaltSysArchitecture, &response);
	if (err != noErr) goto fail;
	
	if (response == gestalt68k)
		ApplyPatches(MenuSelect68k, MenuKey68k, SystemMenu68k);
	else if (response == gestaltPowerPC)
		err = PatchPPC();
	else err = gestaltUnknownErr;	// who knows what might be next?
	
	if (err != noErr) goto fail;
	
	//
	// make sure the INIT stays in memory
	//
	
	DetachResource(hINIT);
	
	goto exit;
	
fail:	
	// we get here if we fail
	
	//
	// deallocate our global memory if it exists
	//
	
	if (gGlobalsPtr) {
		DisposePtr((Ptr)gGlobalsPtr);
		gGlobalsPtr = nil;
	}
	
	//
	// at this point our INIT is not detached, therefore, when our INIT file is closed
	// the resource will automatically be released.
	//
	
	/*
	if (hINIT) {
		ReleaseResource(hINIT);
		hINIT = nil;
	}
	*/
	
	//
	// alert the user to the problem
	//
	
	if (err != noErr) {
		SysBeep(0);
	}
	
exit:	
	// we get here upon success or failure

	//
	// restore a4
	//
	
	SetA4(oldA4);
	
	//
	// restore zone
	//
	
	SetZone(savedZone);
}

/********************************************************************
	RememberMe
	
********************************************************************/

OSErr RememberMe(FSSpecPtr fsp)
{
	FCBPBRec        pb;
	OSErr           err = noErr;
	
	pb.ioCompletion = nil;
	pb.ioNamePtr = fsp->name;
	pb.ioVRefNum = 0;
	pb.ioRefNum = CurResFile();
	pb.ioFCBIndx = 0;
	
	err = PBGetFCBInfoSync(&pb);

	fsp->vRefNum = pb.ioFCBVRefNum;
	fsp->parID = pb.ioFCBParID;
	
	return err;
}

/*************************************************************************

	MenuKey68k

*************************************************************************/

pascal long MenuKey68k(short ch)
{
	long		result;
	long		oldA4;
	OSErr		err = noErr;
	
	//
	// to access our globals
	//
	
	oldA4 = SetUpA4();
	
	//
	// call the original MenuSelect first to see what was selected
	//
	
	result = ((MenuKeyProc)gGlobalsPtr->oldMenuKeyAddr)(ch);
	
	//
	// examine the menu selection
	//
	
	if (result != 0L) {
		err = DoMenuPatchStuff(result, gGlobalsPtr);
		if (err == noErr) 
			result = 0L;
	}
	
	//
	// restore a4 and return
	//
	
	RestoreA4(oldA4);
	return result;
}

/*************************************************************************

	MenuSelect68k

*************************************************************************/

pascal long MenuSelect68k(Point startPt)
{
	long		result;
	long		oldA4;
	OSErr		err = noErr;
	
	//
	// to access our globals
	//
	
	oldA4 = SetUpA4();
	
	//
	// call the original MenuSelect first to see what was selected
	//
	
	result = ((MenuSelectProc)gGlobalsPtr->oldMenuSelectAddr)(startPt);
	
	//
	// examine the menu selection
	//
	
	if (result != 0L) {
		err = DoMenuPatchStuff(result, gGlobalsPtr);
		if (err == noErr) 
			result = 0L;
	}
	
	//
	// restore a4 and return
	//
	
	RestoreA4(oldA4);
	return result;
}

/*************************************************************************

	SystemMenu68k

*************************************************************************/

pascal 	void SystemMenu68k(long menuResult)
{
	long		oldA4;
	OSErr		err = noErr;
	
	//
	// to access our globals
	//
	
	oldA4 = SetUpA4();
	
	//
	// examine the menu selection
	//
	
	if (menuResult != 0L) {
		err = DoMenuPatchStuff(menuResult, gGlobalsPtr);
		if (err == noErr) 
			menuResult = 0L;
	}

	//
	// call the original SystemMenu if we didnt handle it
	//
	
	if (((err == fnfErr) || (err == noErr)) && (menuResult != 0L))
		((SystemMenuProc)gGlobalsPtr->oldSystemMenuAddr)(menuResult);
	
	//
	// restore a4 and return
	//
	
	RestoreA4(oldA4);
}

/*************************************************************************

	ApplyPatches

*************************************************************************/

void ApplyPatches(void * menuSelectCode, void * menuKeyCode, void * systemMenuCode)
{
	gGlobalsPtr->oldMenuSelectAddr = GetToolTrapAddress(_MenuSelect);
	SetToolTrapAddress((UniversalProcPtr)menuSelectCode, _MenuSelect);
	
	gGlobalsPtr->oldMenuKeyAddr = GetToolTrapAddress(_MenuKey);
	SetToolTrapAddress((UniversalProcPtr)menuKeyCode, _MenuKey);
	
	gGlobalsPtr->oldSystemMenuAddr = GetToolTrapAddress(_SystemMenu);
	SetToolTrapAddress((UniversalProcPtr)systemMenuCode, _SystemMenu);
}

/*************************************************************************

	PatchPPC

*************************************************************************/
extern pascal UniversalProcPtr _NewRoutineDescriptor(ProcPtr theProc, ProcInfoType theProcInfo, ISAType theISA)
 TWOWORDINLINE(0x7000, 0xAA59);
extern pascal UniversalProcPtr _NewFatRoutineDescriptor(ProcPtr theM68kProc, ProcPtr thePowerPCProc, ProcInfoType theProcInfo)
 TWOWORDINLINE(0x7002, 0xAA59);

OSErr PatchPPC(void)
{
	OSErr				err = noErr;
	Handle				ppcCodeH = nil;
	SymClass			symClass;
	Ptr					symAddr;
	ConnectionID		connID = kNoConnectionID;
	Str255				errStr;
	Ptr					mainAddr;
	UniversalProcPtr	menuSelectUPP = nil, menuKeyUPP = nil, systemMenuUPP = nil;
	
	//
	// load the PPC resource-based code
	//
	
	ppcCodeH = Get1Resource(kPPCResourceType, kPPCResourceID);
	if (ppcCodeH == nil) return ResError() ? ResError() : resNotFound;
	DetachResource(ppcCodeH);
	HLock(ppcCodeH);
	
	//
	// open a connection to the code fragment
	//
	
	errStr[0] = 0;
	err = GetMemFragment(*ppcCodeH, GetHandleSize(ppcCodeH), kPPCFragmentName,
						 kLoadNewCopy, &connID, &mainAddr, errStr);
	if (err != noErr) goto fail;
	
	//
	// find global data variable
	//
	
	err = FindSymbol(connID, kGlobalsSymbolName, &symAddr, &symClass);
	if (err != noErr) goto fail;
	
	//
	// point the PPC globals to our globals that we have already initialized
	//
	
	*(Globals**)symAddr = gGlobalsPtr;
	
	//
	// get MenuSelect address
	//
	
	err = FindSymbol(connID, kMenuSelectFunctionName, &symAddr, &symClass);
	if (err != noErr) goto fail;
#if 1
	menuSelectUPP = (UniversalProcPtr)_NewRoutineDescriptor((ProcPtr)symAddr, kMenuSelectProcInfo, kPowerPCISA);
#else
	menuSelectUPP = (UniversalProcPtr)_NewFatRoutineDescriptor(MenuSelect68k, symAddr, kMenuKeyProcInfo);
#endif
	if (menuSelectUPP == (UniversalProcPtr)symAddr) DebugStr("\p No RoutineDescriptor");
	if (menuSelectUPP == nil) goto fail;

	//
	// get MenuKey address
	//
	
	err = FindSymbol(connID, kMenuKeyFunctionName, &symAddr, &symClass);
	if (err != noErr) goto fail;
#if 1
	menuKeyUPP = (UniversalProcPtr)_NewRoutineDescriptor((ProcPtr)symAddr, kMenuKeyProcInfo, kPowerPCISA);
#else
	menuKeyUPP = (UniversalProcPtr)_NewFatRoutineDescriptor(MenuKey68k, symAddr, kMenuKeyProcInfo);
#endif
	if (menuKeyUPP == (UniversalProcPtr)symAddr) DebugStr("\p No RoutineDescriptor");
	if (menuKeyUPP == nil) goto fail;
	
	//
	// get SystemMenu address
	//
	
	err = FindSymbol(connID, kSystemMenuFunctionName, &symAddr, &symClass);
	if (err != noErr) goto fail;
#if 1
	systemMenuUPP = (UniversalProcPtr)_NewRoutineDescriptor((ProcPtr)symAddr, kSystemMenuProcInfo, kPowerPCISA);
#else
	systemMenuUPP = (UniversalProcPtr)_NewFatRoutineDescriptor(SystemMenu68k, symAddr, kSystemMenuProcInfo);
#endif
	if (systemMenuUPP == (UniversalProcPtr)symAddr) DebugStr("\p No RoutineDescriptor");
	if (systemMenuUPP == nil) goto fail;
	
	//
	// apply the patches
	//
	
	ApplyPatches(menuSelectUPP, menuKeyUPP, systemMenuUPP);
	
	goto exit;
fail:
	if (menuSelectUPP) {
		DisposeRoutineDescriptor(menuSelectUPP);
		menuSelectUPP = nil;
	}
	if (menuKeyUPP) {
		DisposeRoutineDescriptor(menuKeyUPP);
		menuKeyUPP = nil;
	}
	if (connID != kNoConnectionID) {
		CloseConnection(&connID);
		connID = kNoConnectionID;
	}
	if (ppcCodeH) {
		DisposeHandle(ppcCodeH);
		ppcCodeH = nil;
	}
	if (errStr[0] != 0)
		DebugStr(errStr);
exit:
	return err;
}

