/*
	File Name:	common.c
*/

/*** includes ***********************************************************/

#include <LowMem.h>
#include <ToolUtils.h>
#include <OSUtils.h>
#include <Dialogs.h>
#include <Files.h>
#include <Menus.h>
#include <Script.h>
#include <Resources.h>
#include <Types.h>
#include <Memory.h>
#include <GestaltEqu.h>
#include <Errors.h>
#include <Folders.h>
#include <Processes.h>
#include <EPPC.h>
#include <AppleEvents.h>
#include <Aliases.h>
#include <AppleEvents.h>
#include <AEObjects.h>
#include <AEPackObject.h>
#include <AERegistry.h>

#include "common.h"

/*************************************************************************

	DoMenuPatchStuff

*************************************************************************/

OSErr DoMenuPatchStuff(long menuResult, GlobalsPtr gp)
{
	FSSpec		appFolderSpec;
	OSErr		err = noErr;
	short		menuID = HiWord(menuResult);
	short		menuItem = LoWord(menuResult);
	MenuHandle	hMenu = GetMHandle(menuID);
	FSSpec		menuItemSpec;
	Str255		itemString;
	
	if (hMenu) {
		GetItem(hMenu, menuItem, itemString);
	} else { 
	//	ShowErrorAlert(&gp->ourFSpec, aGetMenuHandle, -1); 
		return -1; 
	}

	//
	// see if a folder exists for the current application
	//
	
	err = FSMakeFSSpec(-1, gp->menuScriptsFolderDirID, (unsigned char *)LMGetCurApName(), &appFolderSpec);
	if (err == noErr) {
		long	appFolderDirID;
		
		//
		// get directory id of current apps folder
		//
		
		err = GetDirectoryID(&appFolderSpec, &appFolderDirID);
		if (err == noErr) {
			
			//
			// make spec for potential item to launch within the current apps folder
			//
			
			err = FSMakeFSSpec(-1, appFolderDirID, itemString, &menuItemSpec);
			if (err == noErr) {
				err = OpenSelection(&menuItemSpec, kOpenItem);
				if (err != noErr) {
					ShowErrorAlert(&gp->ourFSSpec, aOpenSelection, err);
					return err;
				} else goto exit;
			} // fnfErr
		} else { ShowErrorAlert(&gp->ourFSSpec, aGetAppDirID, err); return err; }
	} // fnfErr
	
	//
	// if we are still here, check the Any Application folder
	//
	
	err = FSMakeFSSpec(-1, gp->anyApplicationFolderDirID, itemString, &menuItemSpec);
	if (err == noErr) {
		err = OpenSelection(&menuItemSpec, kOpenItem);
		if (err != noErr)
			ShowErrorAlert(&gp->ourFSSpec, aOpenSelection, err);
		else goto exit;
	} // fnfErr

exit:
	return err;
}

/***************************************************************************

	GetDirectoryID
	
***************************************************************************/

OSErr GetDirectoryID(FSSpec *spec, long *dirID)
{
	OSErr			err = noErr;
	CInfoPBRec		pb;
	
	pb.hFileInfo.ioNamePtr = spec->name;
	pb.hFileInfo.ioVRefNum = spec->vRefNum;
	pb.hFileInfo.ioFDirIndex = 0;
	pb.hFileInfo.ioDirID = spec->parID;

	err = PBGetCatInfoSync(&pb);
	
	*dirID = pb.hFileInfo.ioDirID;
	
	return err;
}

/*************************************************************************

	ShowErrorAlert

*************************************************************************/

#define alrtErrorID	128

void ShowErrorAlert(FSSpecPtr fsp, short action, OSErr err)
{
	Str255	errorString;
	Str32	errorNumStr;
	Str15	emptyString;
	short	fRefNum = -1;
	
	fRefNum = FSpOpenResFile(fsp, fsRdPerm);
	if (fRefNum != -1) {
		emptyString[0] = 0;
		NumToString((long)err, errorNumStr);
		GetIndString(errorString, rErrorStringListID, action);
		ParamText(errorString, errorNumStr, emptyString, emptyString);
		StopAlert(alrtErrorID, nil);
		ParamText(emptyString, emptyString, emptyString, emptyString);
		CloseResFile(fRefNum);
	} else SysBeep(0);
}

/*************************************************************************

	OpenSelection
	
	OpenSelection prepares and sends the Finder OpenSelection event
	This is a hacked down version of OpenSelection() from the DTS Sampler "FinderOpenSel 1.0.1"
	The original version has several options, I've stripped out the ones I don't need and left
	just the code to tell the finder to open the directory window containing the incoming file

*************************************************************************/

#define aeSelectionKeyword	'fsel'
#define aeOpenSelection 	'sope'
#define kFinderSig 			'FNDR'
#define kSystemType 		'MACS'

OSErr OpenSelection(FSSpecPtr fsp, Boolean openOption)
{
    AppleEvent 			aeEvent, aeReply;
    AEDesc 				aeDirDesc, listElem, myAddressDesc;
    FSSpec 				dirSpec;
    AEDesc 				fileList;
    OSErr 				err = noErr;
    ProcessSerialNumber process;
    AliasHandle 		dirAlias, fileAlias;
    
	//
	// get the process serial number of the Finder
	//
	
	err = GetPSN(kSystemType, kFinderSig, &process);
	
	if (err == noErr) {
		
		//
		// create an address descriptor
		//
		
        err = AECreateDesc(typeProcessSerialNumber, (Ptr)&process, sizeof(process), &myAddressDesc);
            
        if (err == noErr) {
			
			//
			// create the apple event
			//
			            
            err = AECreateAppleEvent(kFinderSig, aeOpenSelection, &myAddressDesc, kAutoGenerateReturnID, kAnyTransactionID, &aeEvent);

			//
            // no need to keep this address desc around anymore
            //
            
            AEDisposeDesc(&myAddressDesc);
            
            if (err == noErr) {
                                
				//
				// make a spec to the parent folder
				//
				
                err = FSMakeFSSpec(fsp->vRefNum, fsp->parID, nil, &dirSpec);
                err = NewAlias(nil, &dirSpec, &dirAlias);
				
				//
				// we are either opening the item or showing it in the Finder
				//
				
                if (openOption == kOpenItem)
	                err = NewAlias(nil, fsp, &fileAlias);
                else if (openOption == kShowItem)
                	// since we are opening a window, we just make the file alias the same as the dir alias
               		err = NewAlias(nil, &dirSpec, &fileAlias);
				
				//
				// create list of items to open
				//
				
                if (err == noErr) {
                    err = AECreateList(nil, 0, false, &fileList);
                    
					//
					// create parent folder descriptor
					//
					 
                    HLock((Handle)dirAlias);
                    AECreateDesc(typeAlias, (Ptr)*dirAlias, GetHandleSize((Handle)dirAlias), &aeDirDesc);
                    HUnlock((Handle)dirAlias);
                    if ((err = AEPutParamDesc(&aeEvent, keyDirectObject, &aeDirDesc)) == noErr) {
						
                        AEDisposeDesc(&aeDirDesc);
                        
                        //
                       	// create the file descriptor and add to aliasList
                        //
                        
                        HLock((Handle)fileAlias);
                        AECreateDesc(typeAlias, (Ptr)*fileAlias, GetHandleSize((Handle)fileAlias), &listElem);
                        HLock((Handle)fileAlias);
                        err = AEPutDesc(&fileList, 0, &listElem);
                    }
                    
                    if (err == noErr) {
                        
                        AEDisposeDesc(&listElem);
                        
                        //
                        // add the aliasList to the event
                        //
                        
                        err = AEPutParamDesc(&aeEvent, aeSelectionKeyword, &fileList);
                        AEDisposeDesc(&fileList);
                        
                        if (err == noErr)
                            err = AESend(&aeEvent, &aeReply, kAENoReply + kAEAlwaysInteract + kAECanSwitchLayer, kAENormalPriority, kAEDefaultTimeout, nil, nil);
                    }
                }
                AEDisposeDesc(&aeEvent);
            }
        }
		
	    if ((Handle)dirAlias)
	        DisposHandle((Handle)dirAlias);
	    if ((Handle)fileAlias)
	        DisposHandle((Handle)fileAlias);
	}
	
    return err;
}

/********************************************************************

	GetPSN
	
	returns the process serial number of the file that is currently
	running with given creator and type. if the returns PSN is all
	zeros then it was not found.
	
********************************************************************/

OSErr GetPSN(OSType creator, OSType type, ProcessSerialNumber *process)
{
	ProcessSerialNumber processSerNum;
	ProcessSerialNumber returnProcessSerNum;
	ProcessInfoRec		processInfo;
	Str32				processName;
	FSSpec				fsSpec;
	
	returnProcessSerNum.highLongOfPSN = 0L;
	returnProcessSerNum.lowLongOfPSN = 0L;

	/* start from the beginning */
	processSerNum.highLongOfPSN = 0L;
	processSerNum.lowLongOfPSN = kNoProcess;
	
	processInfo.processInfoLength = sizeof(ProcessInfoRec);
	processInfo.processName = (StringPtr)&processName;
	processInfo.processAppSpec = &fsSpec;
	
	while (GetNextProcess(&processSerNum) == noErr) {
		if (GetProcessInformation(&processSerNum, &processInfo) == noErr) {
			if ((processInfo.processType == (long)type) &&
				(processInfo.processSignature == creator)) {
				BlockMove(&processSerNum, process, sizeof(ProcessSerialNumber));
				return noErr;	
			}
		}
	}
	
	return -1;
}
