/*
	File Name:	cdev.c
*/

#include "cdev.h"
#include "dlogutils.h"
#include "allcommon.h"
#include <GestaltEqu.h>
#include <Processes.h>
#include <Icons.h>

#ifndef __powerc
#include "A4Stuff.h"
#include "SetupA4.h"
#endif

/********************************************************************

	main
	
	procInfo = 1043120 decimal
	
********************************************************************/

pascal long main(	short 		message, 
					short 		item, 
					short 		numItems, 
					short 		CPanelID,
	          		EventRecord *theEvent, 
	          		Handle 		cdevStorage, 	// private storage
	          		DialogPtr	CPDialog)
{
	long	result = 0L;
	
#ifndef __powerc	
	long	oldA4;
	oldA4 = SetCurrentA4();
	RememberA4();
#endif

	if (message == macDev) {		/* check our configuration */
		result = CanRun();
		goto exit;		
	} else if (cdevStorage != nil) {
		
		switch(message) {
		
			/* init ourselves */
			case initDev: 
				cdevStorage = InitControlPanel(CPDialog, numItems);
				if (cdevStorage == nil) {
					result = (long)cdevMemErr;
					goto exit;
				}
				break;
				
			/* close ourselves down - cant depend on the item list here! */
			case closeDev: 
				if (cdevStorage != nil) {
					CloseControlPanel(cdevStorage);
					cdevStorage = nil;
				}
				break;
				
			/* handle hit on item & update cdevStorage each time since 
			   we can not trust the item list during a closeDev msg */
			case hitDev: 
				HitControlPanel(CPDialog, item, numItems, cdevStorage);
				break;
							
			case macDev:		/* check our configuration, ignore it in here though */
			case nulDev:		/* null event */
				break;
				
			case cursorDev:		/* adjust our cursor */
				break;
				
			case updateDev:		/* handle any update drawing */
				UpdatePanel(CPDialog, cdevStorage, numItems);
				break;
				
			case activDev:		/* activate any needed items */
			case deactivDev:	/* deactivate any needed items */
			case keyEvtDev:		/* respond to keydown */
				
				break;
				
			case undoDev:		/* undo event */
			case cutDev:		/* cut event */
			case copyDev:		/* copy event */
			case pasteDev:		/* paste event */
			case clearDev:		/* clear event */
				break;
		}
		
		result = (long)cdevStorage;
		goto exit;
		
	} else {
	
		/* 
		**	if cdevStorage = NIL then ControlPanel 
		**  will put up memory error
		*/
		
		result = nil; /* cdevStorage == nil */
		goto exit;
	}

exit:
	
#ifndef __powerc
	SetA4(oldA4);
#endif

	return result;
}

/********************************************************************

	HitControlPanel
	
********************************************************************/

void HitControlPanel(DialogPtr d, short item, short numItems, Handle cdevStorage)
{
	short myItem = item - numItems;
	switch (myItem) {
		case itemIcon:
			Alert(129, nil);
			break;
		case itemOnRadio:
		case itemOffRadio:
			DoRadioGroup(d, itemOnRadio + numItems, itemOffRadio + numItems, item);
			(*(PreferencesHdl)cdevStorage)->fEnabled = (GetRadioFromGroup(d, itemOnRadio + numItems, itemOffRadio + numItems) == (itemOnRadio + numItems));
			UpdateINIT((PreferencesHdl)cdevStorage);
			break;
	}
}	

/********************************************************************

	CloseControlPanel
	
********************************************************************/

void CloseControlPanel(Handle cdevStorage)
{
	ChangedResource(cdevStorage);
	WriteResource(cdevStorage);
	ReleaseResource(cdevStorage);
}

/********************************************************************

	InitControlPanel
	
********************************************************************/

Handle InitControlPanel(DialogPtr d, short numItems)
{
	Handle 	cdevStorage = nil;
	short	iType;
	Rect	iRect;
	Handle	iHandle;
	
	cdevStorage = Get1Resource(kPreferencesResType, kPreferencesResID);
	if (cdevStorage != nil) {
		DoRadioGroup(d, itemOnRadio + numItems, itemOffRadio + numItems, (*(PreferencesHdl)cdevStorage)->fEnabled ? itemOnRadio : itemOffRadio + numItems);
		GetDItem(d, numItems + itemStaticText, &iType, &iHandle, &iRect);
	#ifdef __powerc
		SetIText(iHandle, "\pThis cdev is native PowerPC code.");
	#else
		SetIText(iHandle, "\pThis cdev is 680x0 code.");
	#endif
	}
	
	return cdevStorage;
}

/********************************************************************

	CanRun
	
********************************************************************/

long CanRun(void)
{
	long	response;
	OSErr	err = noErr;
	
	err = Gestalt(gestaltSystemVersion, &response);
	if ((err != noErr) || (response < 0x00000700)) {
		return (long)false;
	} else {
		return (long)true;
	}
}

/********************************************************************

	UpdatePanel
	
********************************************************************/

void UpdatePanel(DialogPtr d, Handle prefs, short numItems)
{
	GrafPtr		savePort;
	
	/* set up port to draw */
	GetPort(&savePort);
	SetPort(d);
	
	/* draw here */
	
	/* restore port */
	SetPort(savePort);
}

/********************************************************************

	UpdateINIT
	
********************************************************************/

void UpdateINIT(PreferencesHdl ph)
{
	OSErr			err = noErr;
	PreferencesPtr	pp;
	
	err = Gestalt(kGestaltSelector, (long*)&pp);
	if (err == noErr) {
		pp->fEnabled = (*ph)->fEnabled;
	} else SysBeep(0);	// INIT is not loaded, most likely
}
