/*
	File Name:	dlogutils.c
*/

#include "dlogutils.h"

/*******************************************************************************

	SetDialogItemState

*******************************************************************************/

void SetDialogItemState(DialogPtr dlg, short controlNumber, short value)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, controlNumber, &iKind, &iHandle, &iRect);
	SetCtlValue((ControlHandle) iHandle, value);
}

/*******************************************************************************

	GetDialogItemState

*******************************************************************************/

short GetDialogItemState(DialogPtr dlg, short controlNumber)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, controlNumber, &iKind, &iHandle, &iRect);
	return GetCtlValue((ControlHandle) iHandle);
}

/*******************************************************************************

	DoRadioGroup

*******************************************************************************/

void DoRadioGroup(DialogPtr dlg, short lo, short hi, short on)
{
	short i;
	
	for (i=lo;i<=hi;++i)
		SetDialogItemState(dlg, i, 0);
	SetDialogItemState(dlg, on, 1);
}

/*******************************************************************************

	GetRadioFromGroup

*******************************************************************************/

short GetRadioFromGroup(DialogPtr dlg, short lo, short hi)
{
	short i;
	
	for (i=lo;i<=hi;++i) {
		if (GetDialogItemState(dlg, i))
			return i;
	}
	return 0;
}

/*******************************************************************************

	ToggleCheckBox

*******************************************************************************/

void ToggleCheckBox(DialogPtr dlg, short buttonNumber)
{
	short newState;
	
	if (GetDialogItemState(dlg, buttonNumber) == 0)
		newState = 1;
	else
		newState = 0;
	SetDialogItemState(dlg, buttonNumber, newState);
}

/*******************************************************************************

	SetDItemType

*******************************************************************************/

void SetDItemType(DialogPtr dlg, short itemNumber, short newType)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	SetDItem(dlg, itemNumber, newType, iHandle, &iRect);
}

/*******************************************************************************

	EnableControl

*******************************************************************************/

void EnableControl(DialogPtr dlg, short controlNumber)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, controlNumber, &iKind, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 0);
}

/*******************************************************************************

	DisableControl

*******************************************************************************/

void DisableControl(DialogPtr dlg, short controlNumber)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, controlNumber, &iKind, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 255);
}

/*******************************************************************************

	NumberCharacters

*******************************************************************************/

short NumberCharacters(DialogPtr dlg, short itemNumber)
{
	Str255	textStr;
	short	result;
	
	GetItemText(dlg, itemNumber, textStr);
	result = textStr[0];
	return result;
}

/*******************************************************************************

	GetItemText

*******************************************************************************/

void GetItemText(DialogPtr dlg, short itemNumber, unsigned char* textStr)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	GetIText(iHandle, textStr);
}

/*******************************************************************************

	SetItemText

*******************************************************************************/

void SetItemText(DialogPtr dlg, short itemNumber, unsigned char* textStr)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	SetIText(iHandle, textStr);
}

/*******************************************************************************

	GetItemOSType

*******************************************************************************/

OSType GetItemOSType(DialogPtr dlg, short itemNumber)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	Str255	textStr;
	OSType	osType;
	
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	GetIText(iHandle, textStr);
	
	BlockMove(&textStr[1], &osType, sizeof(OSType));
	return osType;
}

/*******************************************************************************

	SetItemOSType

*******************************************************************************/

void SetItemOSType(DialogPtr dlg, short itemNumber, OSType osType)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	Str255	textStr;
	
	BlockMove(&osType, &textStr[1], sizeof(OSType));
	textStr[0] = sizeof(OSType);
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	SetIText(iHandle, textStr);
}

/*******************************************************************************

	GetItemNumber

*******************************************************************************/

unsigned long GetItemNumber(DialogPtr dlg, short itemNumber)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	Str255	textStr;
	long	result;
	
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	GetIText(iHandle, textStr);
	StringToNum(textStr, &result);
	return result;
}

/*******************************************************************************

	SetItemNumber

*******************************************************************************/

void SetItemNumber(DialogPtr dlg, short itemNumber, unsigned long number)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	Str255	textStr;
	
	NumToString(number, textStr);
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	SetIText(iHandle, textStr);
}

/************************************************************************* 
	
	SetWindowFont
	
**************************************************************************/

void SetWindowFont(DialogPtr d, short fontNum, short fontSize, Style fontStyle, short fontMode)
{
	FontInfo	fInfo;
	GrafPtr		savePort;
	
	GetPort(&savePort);
	SetPort(d);
	
	TextFont(fontNum);
    TextSize(fontSize);
    TextFace(fontStyle);
    TextMode(fontMode);
   	
   	GetFontInfo(&fInfo);

    (*((DialogPeek)d)->textH)->fontAscent = fInfo.ascent;
    (*((DialogPeek)d)->textH)->lineHeight = fInfo.ascent + fInfo.descent + fInfo.leading;
    (*((DialogPeek)d)->textH)->txFont = fontNum;
    (*((DialogPeek)d)->textH)->txFace = fontStyle;
    (*((DialogPeek)d)->textH)->txMode = fontMode;
    (*((DialogPeek)d)->textH)->txSize = fontSize;
    
    SetPort(savePort);
}

