/*
	File Name:	ViewByNameLDEF.c
*/

#include <Icons.h>

#include "ViewByNameLDEF.h"
#include "RefConLDEF.h"

RefconLDEFDrawProcUPP 	gListElementProcUPP = nil;
DialogPtr				gd = nil;
ListHandle				glh = nil;

void main(void)
{
	unsigned long	randSeed;
	Boolean			fDone = false;
	GrafPtr			savePort;
	Rect			rView, rDataBnds = {0, 0, kNumRows, kNumColumns};
	Point			cellSize = {kCellHeight, 0};
	UserItemUPP		drawFrameUPP = nil;
	
	// initialize the Mac
	InitGraf(&qd.thePort);
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	GetDateTime(&randSeed);
	LMSetRndSeed(randSeed);
	DrawMenuBar();
	
	// allocate global UPP for our draw proc
	gListElementProcUPP = NewRefconLDEFDrawProc(ListElementProc);
	if (gListElementProcUPP == nil) goto exit;
	
	// allocate local UPP to frame our list user item
	drawFrameUPP = NewUserItemProc(DrawFrame);
	if (drawFrameUPP == nil) goto exit;
	
	// get our main dialog box
	gd = GetNewDialog(dlogMainID, nil, (WindowPtr)-1);
	if (gd == nil) goto exit;
	
	// set the port, font, and user item proc
	GetPort(&savePort);
	SetPort(gd);
	SetWindowFont(gd, geneva, 9, 0, srcCopy);
	SetUserItemProc(gd, itemList, (ProcPtr)drawFrameUPP);
	
	// create the list with one column
	GetDialogItemRect(gd, itemList, &rView);
	rView.right -= kScrollBarWidth;
	glh = LNew(&rView, &rDataBnds, cellSize, kRefConLDEFID, gd, true, false, false, true);
	if (glh == nil) goto exit;

	// show the window
	ShowWindow(gd);
	
	// fill the list with the contents of the root directory
	FillList(-1, fsRtDirID, glh, kAddToEnd, 0);
	LUpdate(gd->visRgn, glh);
	
	// loop until done
	while (!fDone) {
		short	itemHit;
		ModalDialog(nil, &itemHit);
		switch(itemHit) {
		
			case itemQuit:
				fDone = true;
				break;
				
			case itemList:
				HandleClickOnList(gd, glh);
				break;
				
			default:
				break;
		}
	}

exit:	
	// clean up
	if (glh) LDispose(glh);
	if (gd) DisposeDialog(gd);
	if (gListElementProcUPP) DisposeRoutineDescriptor(gListElementProcUPP);
	if (drawFrameUPP) DisposeRoutineDescriptor(drawFrameUPP);
	SetPort(savePort);
}

/***************************************************************************

	FillList
		
***************************************************************************/

void FillList(short vRefNum, long dirID, ListHandle theList, short beforeThisRow, short indentLevel)
{
	short			index;
	OSErr			err = noErr;
	CInfoPBRec		pb;
	ListItemHdl		lih = nil;
	RefconLDEFCell	cellData;
	Cell			cell = {0,0};
	short			maxItems = 32000 / sizeof(RefconLDEFCell);	// roughly approximate the max items in the list
	
	// turn off list drawing
	LDoDraw(false, theList);
	
	// loop through the items in the directory we want to add
	index = 0;
	do {
	
		// allocate a list element
		lih = (ListItemHdl)NewHandleClear(sizeof(ListItem));
		if (lih == nil) goto exit;
		HLock((Handle)lih);
		
		// get the file information for the next item in the directory
		++index;
		pb.hFileInfo.ioNamePtr = (*lih)->spec.name;
		pb.hFileInfo.ioVRefNum = vRefNum;
		pb.hFileInfo.ioFDirIndex = index;
		pb.hFileInfo.ioDirID = dirID;
		err = PBGetCatInfoSync(&pb);
		
		if (err == noErr) {
			if (pb.hFileInfo.ioFlAttrib & ioDirMask)	// we have a folder
				(*lih)->iconID = rFolderIcon;			// use the folder icon
			else										// get custom icon id
				(*lih)->iconID = GetIconID(pb.hFileInfo.ioFlFndrInfo.fdType);
			
			// add a row
			cell.v = LAddRow(1, beforeThisRow++, theList);
			
			// set up the cell data
			(*lih)->spec.parID = dirID;
			(*lih)->spec.vRefNum = vRefNum;
			(*lih)->indentLevel = indentLevel;
			(*lih)->isOpen = false;
			
			cellData.refCon = (long)lih;
			cellData.drawProc = gListElementProcUPP;
			
			// set the cell data
			LSetCell(&cellData, sizeof(RefconLDEFCell), cell, theList);
		}
	
		HUnlock((Handle)lih);
		if (err != noErr) DisposeHandle((Handle)lih);

	} while ((err == noErr) && ((**theList).dataBounds.bottom <= maxItems));
	
exit:
	LDoDraw(true, theList);
}

/***************************************************************************

	HandleClickOnList
		
***************************************************************************/

void HandleClickOnList(DialogPtr d, ListHandle theList)
{	
	Point 			localPt = {0,0};
	Boolean			doubleClick = false;
	Rect			cellRect = {0,0,0,0};
	ListItemHdl 	lih = nil;
	RefconLDEFCell	cellData;
	short			dataLen = sizeof(RefconLDEFCell);
	Cell			theCell = {0,0};
	Rect			iconRect, trackRect;
	Boolean			found = false;
	
	// get the location of the mouse click in local coordinates
	GetMouse(&localPt);
	
	// if we are clicking in area of the folder flag icons...
	if (localPt.h < (**theList).rView.left + 20) {
		
		// see if we clicked in any cell in particular
		theCell.h = theCell.v = 0;
		do {
			LRect(&cellRect, theCell, theList);
			if (PtInRect(localPt, &cellRect))
				found = true;
		} while (!found && LNextCell(false, true, &theCell, theList));
		
		// if we did, then see if the cell is a folder by checking the iconID
		if (found) {
			LGetCell(&cellData, &dataLen, theCell, theList);
			lih = (ListItemHdl)cellData.refCon;
			if ((*lih)->iconID == rFolderIcon) {
				iconRect = cellRect;
				iconRect.left += 4;
				iconRect.right = iconRect.left + 16;
				iconRect.top += 1;
				iconRect.bottom = iconRect.top + 16;
				trackRect = cellRect;
				trackRect.right = trackRect.left + 20;
				
				// track the flag icon
				doubleClick = TrackIconByRect(&iconRect, &trackRect, (*lih)->isOpen ? rArrowOpenedIcon : rArrowClosedIcon);
				
				// if the user released within the flag then we do the same as a double click
				if (doubleClick) {
					Cell theTCell;
					
					// we also select the cell that we are about to "open"
					if (AnyCellsSelected(theList, &theTCell))
						LSetSelect(false, theTCell, theList);
					LSetSelect(true, theCell, theList);
				}
			} else {
				// otherwise let LClick tell us if the user double-clicked
				// on a non-folder item and do whatever you like.
				doubleClick = LClick(localPt, 0, theList);
			}
		}
	} else {
		// otherwise let LClick tell us if the user double-clicked on an item .
		doubleClick = LClick(localPt, 0, theList);
	}
	
	// if the user double-clicked on something...				
	if (doubleClick) {
		// find out what
		if (AnyCellsSelected(theList, &theCell)) {
			LGetCell(&cellData, &dataLen, theCell, theList);
			lih = (ListItemHdl)cellData.refCon;
			
			// if it's a folder...
			if ((*lih)->iconID == rFolderIcon) {
				// and it's not opened...
				if ((*lih)->isOpen == false) {
					// get the directory id of it
					long dirID = GetDirectoryID(&((*lih)->spec));
					if (dirID != 0L) {
						// and fill the list with its contents then redraw the list
						FillList((*lih)->spec.vRefNum, dirID, theList, theCell.v + 1, (*lih)->indentLevel + 1);
						(*lih)->isOpen = true;
						LSetCell(&cellData, sizeof(RefconLDEFCell), theCell, theList);
						LUpdate(gd->visRgn, glh);
					}
				} else { 
					// close the folder by deleting list rows below it until we reach the same
					// indentation level as the folder we are closing.
					short	indentLevelToDelete = (*lih)->indentLevel + 1;
					short	numRowsToDelete = 0;
					short	firstRowToDelete = theCell.v + 1;
					
					(*lih)->isOpen = false;
					LSetCell(&cellData, sizeof(RefconLDEFCell), theCell, theList);
					
					// loop checking each cells indentation level
				checkNextCell:
					theCell.v++;
					if (theCell.v < (**theList).dataBounds.bottom) {
						dataLen = sizeof(RefconLDEFCell);
						LGetCell(&cellData, &dataLen, theCell, theList);
						lih = (ListItemHdl)cellData.refCon;
						if ((*lih)->indentLevel >= indentLevelToDelete) {
							numRowsToDelete++;
							DisposeHandle((Handle)lih);
							lih = nil;
							goto checkNextCell;
						}
					}
					
					// once we count how many rows to delete, nuke them all at once
					if (numRowsToDelete > 0L)
						LDelRow(numRowsToDelete, firstRowToDelete, theList);
				}
			}
		}
	}
}

/************************************************************************

	ListElementProc

	This function is called for each cell to draw its contents. By using
	the RefConLDEF we allow the ability to have a different drawing
	procedure for each cell if we need it. In this case however, we
	decide to simply use one simple function to draw each cell.
************************************************************************/

void ListElementProc(Rect *cellRect, Cell lCell, ListHandle theList, long refCon)
{
	ListItemHdl		lih;
	GrafPtr			savePort;
	Rect			iconRect;
	short			width;
	FInfo			fndrInfo;
	
	// typecast our refCon
	lih = (ListItemHdl)refCon;
	
	// set up the port
	GetPort(&savePort);
	SetPort((**theList).port);
	PenNormal();
	
	// draw the file/folder name, in italics if it is an alias
	MoveTo(cellRect->left + (44 + (20 * (*lih)->indentLevel)), cellRect->bottom - 4);
	if (FSpGetFInfo(&((*lih)->spec), &fndrInfo) == noErr) {
		if (fndrInfo.fdFlags & 0x8000)
			TextFace(italic);
		DrawString((*lih)->spec.name);
		TextFace(0);
	} else
		DrawString((*lih)->spec.name);
	width = StringWidth((*lih)->spec.name);
	
	// draw the folder flag icon if need be
	if ((*lih)->iconID == rFolderIcon) {
		iconRect = *cellRect;
		iconRect.left += 4;
		iconRect.right = iconRect.left + 16;
		iconRect.top += 1;
		iconRect.bottom = iconRect.top + 16;
		PlotIconID(&iconRect, atNone, ttNone, (*lih)->isOpen ? rArrowOpenedIcon : rArrowClosedIcon);	
	}
	
	// draw the folder icon or file icon. if the real icon for the file can not be
	// grabbed from the desktop database then we draw the generic version as
	// stored in the iconID field of the cells data
	iconRect = *cellRect;
	iconRect.left += (24 + (20 * (*lih)->indentLevel));
	iconRect.right = iconRect.left + 16;
	iconRect.top += 1;
	iconRect.bottom = iconRect.top + 16;

	if ((*lih)->iconID == rFolderIcon) {
		PlotIconID(&iconRect, atNone, ttNone, (*lih)->iconID);	
	} else {
		if (DrawFileIcon(&((*lih)->spec), &iconRect) != noErr)
			PlotIconID(&iconRect, atNone, ttNone, (*lih)->iconID);	
	}
	
	SetPort(savePort);	
}

#pragma mark -

/*******************************************************************************

	DrawFrame
	
*******************************************************************************/

pascal void DrawFrame(DialogPtr d, short theItem)
{
	Rect 	r;
	GrafPtr	savePort;
	
	GetPort(&savePort);
	SetPort(d);
	GetDialogItemRect(d, theItem, &r);
	InsetRect(&r, -1, -1);
	FrameRect(&r);
	SetPort(savePort);
}

/*******************************************************************************

	GetDialogItemRect

*******************************************************************************/

void GetDialogItemRect(DialogPtr dlg, short itemNumber, Rect *r)
{
	short	iKind;
	Handle	iHandle;
	GetDItem(dlg, itemNumber, &iKind, &iHandle, r);
}

/************************************************************************* 
	
	SetWindowFont
	
**************************************************************************/

void SetWindowFont(DialogPtr d, short fontNum, short fontSize, Style fontStyle, short fontMode)
{
	FontInfo	fInfo;
	GrafPtr		savePort;
	
	GetPort(&savePort);
	SetPort(d);
	
	TextFont(fontNum);
    TextSize(fontSize);
    TextFace(fontStyle);
    TextMode(fontMode);
   	
   	GetFontInfo(&fInfo);

    (*((DialogPeek)d)->textH)->fontAscent = fInfo.ascent;
    (*((DialogPeek)d)->textH)->lineHeight = fInfo.ascent + fInfo.descent + fInfo.leading;
    (*((DialogPeek)d)->textH)->txFont = fontNum;
    (*((DialogPeek)d)->textH)->txFace = fontStyle;
    (*((DialogPeek)d)->textH)->txMode = fontMode;
    (*((DialogPeek)d)->textH)->txSize = fontSize;
    
    SetPort(savePort);
}

/*******************************************************************************

	SetUserItemProc

*******************************************************************************/

void SetUserItemProc(DialogPtr dlg, short userItem, ProcPtr userProc)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;

	GetDItem(dlg, userItem, &iKind, &iHandle, &iRect);
	SetDItem(dlg, userItem, iKind, (Handle)userProc, &iRect);
}

/*******************************************************************************

	ShowError

*******************************************************************************/

void ShowError(OSErr errorCode, unsigned char * errorStr)
{
	Str32	errorCodeStr;
	NumToString((long)errorCode, errorCodeStr);
	ParamText(errorCodeStr, errorStr, "\p", "\p");
	StopAlert(256, nil);
	ParamText("\p", "\p", "\p", "\p");
}

/***************************************************************************

	GetDirectoryID
		
***************************************************************************/

long GetDirectoryID(FSSpec *spec)
{
	OSErr			err;
	CInfoPBRec		pb;
	
	pb.hFileInfo.ioNamePtr = spec->name;
	pb.hFileInfo.ioVRefNum = spec->vRefNum;
	pb.hFileInfo.ioFDirIndex = 0;
	pb.hFileInfo.ioDirID = spec->parID;
	err = PBGetCatInfoSync(&pb);
	if (err == noErr)
		return pb.hFileInfo.ioDirID;
	else
		return 0L;
}

/********************************************************************

	TrackIconByRect
	
********************************************************************/

Boolean TrackIconByRect(Rect *drawRect, Rect *trackRect, short iconID)
{
	Boolean		mouseReleasedWithin = false;
	Point		oldPt, newPt;
	
	oldPt.h = oldPt.v = -1;
	
	if (StillDown()) {
		while (WaitMouseUp()) {
			GetMouse(&newPt);
			if (DeltaPoint(oldPt, newPt)) {
				if (PtInRect(newPt, trackRect)) {
					if (!mouseReleasedWithin)
						PlotIconID(drawRect, atNone, ttSelected, iconID);	/* in */
					mouseReleasedWithin = true;
				} else {
					if (mouseReleasedWithin)
						PlotIconID(drawRect, atNone, ttNone, iconID);		/* out */
					mouseReleasedWithin = false;
				}
				oldPt = newPt;
			}
		}
	}
	if (mouseReleasedWithin)
		PlotIconID(drawRect, atNone, ttNone, iconID);						/* draw "out" as last thing */	
	return mouseReleasedWithin;
}

/***************************************************************************

	GetIconID
		
***************************************************************************/

#define rINITIcon				133
#define rApplicationIcon		134
#define rDocumentIcon			135
#define rTEXTIcon				136
#define rcdevIcon				137
#define rttroIcon				138
#define rdfilIcon				139
#define rsfilIcon				140

short GetIconID(OSType fileType)
{
	switch(fileType) {
		case 'APPL':
			return rApplicationIcon;
			break;
		case 'TEXT':
			return rTEXTIcon;
			break;
		case 'cdev':
			return rcdevIcon;
			break;
		case 'INIT':
			return rINITIcon;
			break;
		case 'ttro':
			return rttroIcon;
			break;
		case 'dfil':
			return rdfilIcon;
			break;
		case 'sfil':
			return rsfilIcon;
			break;
		default:
			return rDocumentIcon;
	}
}

/********************************************************************

	AnyCellsSelected
	
********************************************************************/

Boolean AnyCellsSelected(ListHandle lh, Cell *theCell)
{
	theCell->v = theCell->h = 0;
	return LGetSelect(true, theCell, lh);
}

/***************************************************************************

	DrawFileIcon
		
***************************************************************************/

OSErr DrawFileIcon(FSSpec *spec, Rect *drawRect)
{
	OSErr 	err = noErr;
	Handle	theSuite = NULL;
	DTPBRec	pb;
	short	ioDTRefNum;
	FInfo	fndrInfo;
	Handle	theIconData = NULL;
	
	/* get desktop database reference number */
	pb.ioNamePtr = NULL;
	pb.ioVRefNum = -1; // spec->vRefNum;
	err = PBDTGetPath(&pb);
	if (err != noErr) goto exit;
	ioDTRefNum = pb.ioDTRefNum;

	/* create an icon suite to store icons in */
	err = NewIconSuite(&theSuite);
	if (err != noErr) goto exit;

	/* get file type/creator */
	err = FSpGetFInfo(spec, &fndrInfo);
	if (err != noErr) goto exit;
	
	/* get small bw icon */
	theIconData = NewHandle(kSmallIconSize);
	if (theIconData) {
		HLock(theIconData);
		pb.ioCompletion = NULL;
		pb.ioDTRefNum = ioDTRefNum;
		pb.ioDTBuffer = *theIconData;
		pb.ioDTReqCount = kSmallIconSize;
		pb.ioIconType = kSmallIcon;
		pb.ioFileCreator = fndrInfo.fdCreator;
		pb.ioFileType = fndrInfo.fdType;
		err = PBDTGetIconSync(&pb);
		if (err == noErr) {
			HUnlock(theIconData);
			err = AddIconToSuite(theIconData, theSuite, small1BitMask);
		} else {
			DisposeHandle(theIconData);
			theIconData = NULL;
		}
		if (err != noErr) goto exit;
	}

	/* get small 4 bit icon */
	theIconData = NewHandle(kSmall4BitIconSize);
	if (theIconData) {
		HLock(theIconData);
		pb.ioCompletion = NULL;
		pb.ioDTRefNum = ioDTRefNum;
		pb.ioDTBuffer = *theIconData;
		pb.ioDTReqCount = kSmall4BitIconSize;
		pb.ioIconType = kSmall4BitIcon;
		pb.ioFileCreator = fndrInfo.fdCreator;
		pb.ioFileType = fndrInfo.fdType;
		err = PBDTGetIconSync(&pb);
		if (err == noErr) {
			HUnlock(theIconData);
			err = AddIconToSuite(theIconData, theSuite, small4BitData);
		} else {
			DisposeHandle(theIconData);
			theIconData = NULL;
		}
		if (err != noErr) goto exit;
	}

	/* get small 8 bit icon */
	theIconData = NewHandle(kSmall8BitIconSize);
	if (theIconData) {
		HLock(theIconData);
		pb.ioCompletion = NULL;
		pb.ioDTRefNum = ioDTRefNum;
		pb.ioDTBuffer = *theIconData;
		pb.ioDTReqCount = kSmall8BitIconSize;
		pb.ioIconType = kSmall8BitIcon;
		pb.ioFileCreator = fndrInfo.fdCreator;
		pb.ioFileType = fndrInfo.fdType;
		err = PBDTGetIconSync(&pb);
		if (err == noErr) {
			HUnlock(theIconData);
			err = AddIconToSuite(theIconData, theSuite, small8BitData);
		} else {
			DisposeHandle(theIconData);
			theIconData = NULL;
		}
		if (err != noErr) goto exit;
	}

	/* plot the icons */
	err = PlotIconSuite(drawRect, atNone, ttNone, theSuite);
	if (err != noErr) goto exit;
	
exit:
	/* dispose the suite if need be */
	if (theSuite != NULL)
		DisposeIconSuite(theSuite, true);
	return err;
}
