/*
	File Name:	Tiger Slider.cp
*/

/**************************************************************************************

	Tiger Slider 1.0b2 - 1/9/95
	
	1995 Robert L Mathews, Tiger Technologies
	tigers@tigertech.com
	
	This code may be freely distributed, modified, and used in any project, as long as
	this notice is not removed from the source code. We also request, but do not require,
	that credit be given in your program's documentation or about box if you use it.
	
	The CDEF that this project generates is similar to that in the "Sound" control
	panel (except that ours is horizontal) - it consists of a background picture and
	an "indicator" that slides smoothly over the background as the user drags the mouse
	button after clicking on the indicator. An extra feature we include is the ability
	to click anywhere on the background and have the indicator jump to that position
	immediately.
	
	As written, this CDEF requires System 7, because it requires GWorlds even on Macs
	without Color QuickDraw. (Another note is that it uses C++ style inline variable
	declaration; if you want to compile it with a straight C compiler, you'll have to
	move all the variable declarations to the top of each function.)
	
	This slider uses double-buffered animation. This technique allows animation without
	the flicker you would see if you were to simply erase the screen and redraw the
	new position of the indicator every time it moves. If you were to try this, you
	would soon find that no matter how quickly you draw the new position after
	erasing the old one, the screen displays a distracting flash of white for a moment.
	Instead, we perform the erasing and redrawing in an "offscreen buffer" - an area
	of memory that the user does not see. This area is called a GWorld (for more
	information about GWorlds, see "New Inside Macintosh: Imaging with QuickDraw").
	After the new image has been drawn in the GWorld, we copy the GWorld to the screen
	using CopyBits, which plasters the new image on the screen all at once -
	erasing the old image and drawing the new at the same time. There is never
	a moment where the old indicator is invisible and the new one has not yet been
	drawn.
	
	To do this, we actually use three offscreen GWorlds - the background world, which
	stores a copy of the background of the slider with no indicator on it; the
	indicator world, which stores the indicator (this GWorld will be narrower in terms of
	pixel width); and the composite world, which is the size of the contrlRect of the
	control itself, and is used for temporarily merging the two other worlds.
	
	So. Whenever we need to draw a piece of the slider to the screen (because, for
	instance, the user drags the indicator around), here's what we do:
	
	1.	Figure out the part that needs to be drawn - such as the rectangle enclosing
			both the old and new locations of the indicator.
	2.	Copy that rectangle from the background world to the composite world. This
			section of the composite world now looks just the same section of the
			background world - notice that we've erased the old location here, but not
			on screen, so the user hasn't seen anything yet.
	3.	Draw the indicator in the appropriate new position of the composite world.
			This section of the composite world now looks just like it should on screen,
			but the user still hasn't seen anything.
	4.	Copy this entire rectangle (old and new) from the composite world to the screen
			all at once. The screen has now been updated, with the old location of the
			indicator being erased at exactly the same time the new one is drawn!
	
	This gives pretty much flicker-free drawing, which is what we want. (The only thing
	that might cause a slight flicker is "tearing", which occurs when the electron beam
	of the screen passes through the copied rectangle while we're drawing it. Believe
	it or not, there's a way to prevent this from happening by waiting for the electron
	beam to shut off as it moves from the bottom to the top of screen and drawing [very
	quickly] then, but that's a little complicated for this example!)
	
**************************************************************************************/

#include	"Tiger Slider.h"
#include	<QDOffscreen.h>
#include	<Controls.h>


#ifdef powerc
ProcInfoType __procinfo = uppControlDefProcInfo;
#endif

/**************************************************************************************

	main - this is the entry point called by the system whenever our routine should
	take some action.
	
**************************************************************************************/
pascal long main (short /*varCode*/, ControlHandle theControl, short theMessage, long theParam)
{
	long	result = 0;
	
	switch (theMessage)
	{
		case	initCntl:
			InitTheControl (theControl);
			break;
			
		case	dispCntl:
			DisposeTheControl (theControl);
			break;
		
		case	drawCntl:
			DrawTheControl (theControl);
			break;

		case	testCntl:
			if (PtInRect ((*(Point*) &theParam), &(*theControl) -> contrlRect))
			{
				// tell the Control Manager that a click occurred in some
				// part of the control, and that dragCntl should be called -
				// the number 130 is not particularly special, except that it
				// indicates that the click area is a custom part of ours
				result = 130;	
			}
			break;

		case	dragCntl:
			if (theParam)
			{
				DragTheControl (theControl);
				result = 1;		// signals that we handled the dragging all by ourselves
			}
			break;

		case calcCntlRgn:
		case calcThumbRgn:
			// these messages are received if we are in 32-bit mode
			RectRgn ((RgnHandle) (theParam), &(*theControl) -> contrlRect);
			break;

		case calcCRgns:
			// this message is only received in 24 bit mode; we have to be sure
			// that we clear the high byte (we do so using StripAddress)
			RgnHandle tempRgn = (RgnHandle) StripAddress ((Ptr) theParam);
			RectRgn (tempRgn, &(*theControl) -> contrlRect);
		break;
	}

	return (result);
}


/**************************************************************************************

	InitTheControl - used to allocate the GWorlds and prepare the backgrounds by filling
	them in with pictures.
	
**************************************************************************************/
void InitTheControl (ControlHandle theControl)
{
	OSErr					err;
	SliderInfoH		resHandle;
	
	
	// The info about this slider is stored in a resource of type 'Sinf' that has the same
	// ID as the refcon of this control. Note that the refcon is only used during the
	// creation of the control; the application may use it in any way after that (i.e., after
	// GetNewControl or NewControl has returned). The CDEF, however, does use the contrlData
	// field all the time to keep track of the 'Sinf' handle, so don't mess with contrlData
	// in your application!
	
	(*theControl) -> contrlData = nil;
	resHandle = (SliderInfoH) GetResource (kSliderInfoType, (*theControl) -> contrlRfCon);
	if (!resHandle)
	{
		// the resource was not found - set contrlData to nil so that other functions know that
		// initialization failed! If this happens, no crash will occur, but the CDEF won't do anything.
		(*theControl) -> contrlData = nil;
	}
	else
	{
		(*theControl) -> contrlData = (Handle) resHandle;

		// make a non-purgeable, non-resource copy of the 'Sinf' handle - from now on,
		// "contrlData" always refers to a copy of the 'Sinf'.
		HNoPurge ((*theControl) -> contrlData);
		DetachResource ((*theControl) -> contrlData);
		
		HLock ((*theControl) -> contrlData);		// temporary for this function
		
		Rect compositeRect = (*theControl) -> contrlRect;
		OffsetRect (&compositeRect, -compositeRect.left, -compositeRect.top);

		// allocate the GWorlds - note that we use the bit depth specified in the 'Sinf' resource, rather than
		// matching the depth of the screen. An improvement that can be made here would be to match the depth
		// of the screen that the majority of the control appears on, and use that screen's color table - however,
		// if this change is made, it introduces the necessity of checking to make sure that the user hasn't
		// moved the control to another screen each time it's drawn, and dealing with it (by reallocating the
		// GWorlds and reloading the picture) if so. This task is left as an exercise to the reader - have fun!
		
		// also note that we don't do anything useful here if the GWorld cannot be allocated, but we do check that
		// allocation was successful everywhere else before we use it, so that no crash will occur. You might see
		// some funny things on screen, though! Fortunately, you know how much memory your slider will need (based
		// on the bit depth, as described above) so your program can check for it before creating the control.

		err = NewGWorld (&(*resHandle) -> indicatorWorld, (*resHandle) -> bitDepth, &(*resHandle) -> indicatorRect, nil, nil, 0);
		if (err)
		{
			// not enough memory in the application heap! Try it in temporary memory.
			err = NewGWorld (&(*resHandle) -> indicatorWorld, (*resHandle) -> bitDepth, &(*resHandle) -> indicatorRect, nil, nil, useTempMem);
		}
		
		if (!err)
		{
			GWorldPtr			currPort;
			GDHandle			currDev;

			GetGWorld (&currPort, &currDev);		// save the current GWorld
			SetGWorld ((*resHandle) -> indicatorWorld, nil);		// set to the new one, so that any drawing takes place there
			PixMapHandle pixMap = GetGWorldPixMap ((*resHandle) -> indicatorWorld);
			LockPixels (pixMap);
			
			PicHandle picture = GetPicture ((*resHandle) -> indicatorPictResID);
			if (picture)
			{
				DrawPicture (picture, &(*resHandle) -> indicatorWorld->portRect);
			}
		
			UnlockPixels (pixMap);
			SetGWorld (currPort, currDev);		// restore the original GWorld
		}
		
		err = NewGWorld (&(*resHandle) -> backgroundWorld, (*resHandle) -> bitDepth, &compositeRect, nil, nil, 0);
		if (err)
		{
			// not enough memory in the application heap! Try it in temporary memory.
			err = NewGWorld (&(*resHandle) -> backgroundWorld, (*resHandle) -> bitDepth, &compositeRect, nil, nil, useTempMem);
		}

		if (!err)
		{
			GWorldPtr			currPort;
			GDHandle			currDev;

			GetGWorld (&currPort, &currDev);		// save the current GWorld
			SetGWorld ((*resHandle) -> backgroundWorld, nil);		// set to the new one, so that any drawing takes place there
			PixMapHandle pixMap = GetGWorldPixMap ((*resHandle) -> backgroundWorld);
			LockPixels (pixMap);
			
			PicHandle picture = GetPicture ((*resHandle) -> backgroundPictResID);
			if (picture)
			{
				DrawPicture (picture, &compositeRect);
			}
		
			UnlockPixels (pixMap);
			SetGWorld (currPort, currDev);		// restore the original GWorld
		}
		
		err = NewGWorld (&(*resHandle) -> compositeWorld, (*resHandle) -> bitDepth, &compositeRect, nil, nil, 0);
		if (err)
		{
			// not enough memory in the application heap! Try it in temporary memory.
			NewGWorld (&(*resHandle) -> compositeWorld, (*resHandle) -> bitDepth, &compositeRect, nil, nil, useTempMem);
		}
		
		HUnlock ((*theControl) -> contrlData);
	}
}

/**************************************************************************************

	DisposeTheControl - used to deallocate the GWorlds.
	
**************************************************************************************/
void DisposeTheControl (ControlHandle theControl)
{
	SliderInfoH		sHandle = (SliderInfoH) (*theControl) -> contrlData;
	
	if (sHandle)
	{
		if ((*sHandle) -> indicatorWorld)
		{
			DisposeGWorld ((*sHandle) -> indicatorWorld);
		}
		
		if ((*sHandle) -> backgroundWorld)
		{
			DisposeGWorld ((*sHandle) -> backgroundWorld);
		}
		
		if ((*sHandle) -> compositeWorld)
		{
			DisposeGWorld ((*sHandle) -> compositeWorld);
		}
		
		DisposeHandle ((Handle) sHandle);
	}
}

/**************************************************************************************

	DrawTheControl - technically it's possible to determine whether we need to draw the
	entire control or just the indicator, but it's simpler just to draw the entire
	thing - "just drawing the indicator" doesn't really make much sense under our
	offscreen buffering mechanism.
	
**************************************************************************************/
void DrawTheControl (ControlHandle theControl)
{
	if ((*theControl) -> contrlData)
	{
		HLock ((*theControl) -> contrlData);
		SliderInfoP	infoPtr = (*(SliderInfoH) (*theControl) -> contrlData);
	
		if ((*theControl) -> contrlVis && infoPtr -> indicatorWorld && infoPtr -> backgroundWorld && infoPtr -> compositeWorld)
		{
			GWorldPtr			currPort;
			GDHandle			currDev;
		
			GetGWorld (&currPort, &currDev);		// save the current GWorld
			SetGWorld (infoPtr -> compositeWorld, nil);		// set to the new one, so that any drawing takes place there
			PixMapHandle compositePixMap = GetGWorldPixMap (infoPtr -> compositeWorld);
			LockPixels (compositePixMap);
				
			// first, copy the background to the composite
			PixMapHandle backgroundPixMap = GetGWorldPixMap (infoPtr -> backgroundWorld);
			LockPixels (backgroundPixMap);
		
			CopyBits ((BitMap*) *backgroundPixMap, (BitMap*) *compositePixMap,
									&infoPtr -> backgroundWorld -> portRect, &infoPtr -> compositeWorld -> portRect,
									srcCopy, nil);
	
			UnlockPixels (backgroundPixMap);
	
	
			// now draw the indicator on top of the background image that's in the composite
			PixMapHandle indicatorPixMap = GetGWorldPixMap (infoPtr -> indicatorWorld);
			LockPixels (indicatorPixMap);
		
			// calulate where the indicator should appear
			Rect	destinationRect = infoPtr -> indicatorRect;
			short	offsetUnits = (*theControl) -> contrlValue - (*theControl) -> contrlMin;
			OffsetRect (&destinationRect, (infoPtr -> pixelsPerValue * offsetUnits), 0);
			
			CopyBits ((BitMap*) *indicatorPixMap, (BitMap*) *compositePixMap,
									&infoPtr -> indicatorWorld -> portRect, &destinationRect,
									srcCopy, nil);
	
			UnlockPixels (indicatorPixMap);
	
	
			// now copy the composite to the screen
			SetGWorld (currPort, currDev);						// restore the original GWorld
			
			ForeColor (blackColor);		// avoid unwanted coloring by CopyBits - you have to do this to
			BackColor (whiteColor);		// prevent CopyBits from changing the color of the pixels being copied!
			
			CopyBits ((BitMap*) *compositePixMap, &((GrafPtr) (*theControl) -> contrlOwner) -> portBits,
									&infoPtr -> compositeWorld -> portRect, &(*theControl) -> contrlRect,
									srcCopy, nil);
	
			UnlockPixels (compositePixMap);
		}
		HUnlock ((*theControl) -> contrlData);
	}
}

/**************************************************************************************

	DragTheControl
	
**************************************************************************************/
void DragTheControl (ControlHandle theControl)
{
	if ((*theControl) -> contrlData)
	{
		HLock ((*theControl) -> contrlData);
		SliderInfoP	infoPtr = (*(SliderInfoH) (*theControl) -> contrlData);

		Point			mousePt,					// where the mouse is right now
							topLeftPt;				// where the top left of the control is in local coords
		short			leftEdgeOffset,		// the distance between the mousePt and where
																// the left edge of the indicator should be
							leftLoc,					// ignore mouse coords to the left of this position
							rightLoc,					// ignore mouse coords to the right of this position
							oldMouseHoriz,		// where the mouse was horizontally last time through the loop
							oldValue,					// original value of the control
							newValue;					// new value of the control
		GWorldPtr	currPort;
		GDHandle	currDev;
		Rect			oldThumb,					// only the union of the old and new positions needs to be drawn each
																// time the slider moves, and this rectangle stores the old one
							newThumb,					// the new rectangle for the indicator
							unionOfRects,
							tempRect;
		
		
		GetGWorld (&currPort, &currDev);		// save the current GWorld
		PixMapHandle compositePixMap = GetGWorldPixMap (infoPtr -> compositeWorld);
		LockPixels (compositePixMap);
		
		PixMapHandle backgroundPixMap = GetGWorldPixMap (infoPtr -> backgroundWorld);
		LockPixels (backgroundPixMap);

		PixMapHandle indicatorPixMap = GetGWorldPixMap (infoPtr -> indicatorWorld);
		LockPixels (indicatorPixMap);

		ForeColor (blackColor);		// avoid unwanted coloring by CopyBits
		BackColor (whiteColor);
		
		// calulate where the indicator is to start with
		oldThumb = infoPtr -> indicatorRect;
		short	offsetUnits = (*theControl) -> contrlValue - (*theControl) -> contrlMin;
		OffsetRect (&oldThumb, (infoPtr -> pixelsPerValue * offsetUnits), 0);

		SetPt (&topLeftPt, (*theControl) -> contrlRect.left, (*theControl) -> contrlRect.top);
		oldMouseHoriz = -1;		// ensure we always draw at least once
	
		leftEdgeOffset = (infoPtr -> indicatorRect.right - infoPtr -> indicatorRect.left) / 2;
		leftLoc = infoPtr -> indicatorRect.left + leftEdgeOffset;
		rightLoc = leftLoc + (infoPtr -> pixelsPerValue * ((*theControl) -> contrlMax - (*theControl) -> contrlMin));
		
		oldValue = (*theControl) -> contrlValue,
		newValue = oldValue;
		
		do		// always at least once - otherwise the user might click and release the mouse before we started tracking;
					// we want to be sure we jump to the new value if this happens
		{
			GetMouse (&mousePt);
			mousePt.h -= topLeftPt.h;
			mousePt.v -= topLeftPt.v;
			
			mousePt.h = min (mousePt.h, rightLoc);
			mousePt.h = max (mousePt.h, leftLoc);
	
			if (mousePt.h != oldMouseHoriz)
			{
				newThumb = oldThumb;
				newThumb.left = mousePt.h - leftEdgeOffset;
				newThumb.right = newThumb.left + (infoPtr -> indicatorRect.right - infoPtr -> indicatorRect.left);

				UnionRect (&oldThumb, &newThumb, &unionOfRects);
				SetGWorld (infoPtr -> compositeWorld, nil);

				CopyBits ((BitMap*) *backgroundPixMap, (BitMap*) *compositePixMap,
										&unionOfRects, &unionOfRects, srcCopy, nil);
				
				CopyBits ((BitMap*) *indicatorPixMap, (BitMap*) *compositePixMap,
										&infoPtr -> indicatorWorld -> portRect, &newThumb, srcCopy, nil);
				
				SetGWorld (currPort, currDev);		// restore the original GWorld
				tempRect = unionOfRects;
				OffsetRect (&tempRect, topLeftPt.h, topLeftPt.v);
				CopyBits ((BitMap*) *compositePixMap, &((GrafPtr) (*theControl) -> contrlOwner) -> portBits,
										&unionOfRects, &tempRect, srcCopy, nil);
				
				oldThumb = newThumb;
				oldMouseHoriz = mousePt.h;
				
				short rawValue = newThumb.left - infoPtr -> indicatorRect.left;
				newValue = rawValue / infoPtr -> pixelsPerValue;
				if ((rawValue % infoPtr -> pixelsPerValue) > (infoPtr -> pixelsPerValue / 2))
				{
					// it's more than halfway to the next "stop"
					newValue ++;
				}
			}
		}	while (StillDown ());
		
		UnlockPixels (indicatorPixMap);
		UnlockPixels (backgroundPixMap);
		UnlockPixels (compositePixMap);
		SetGWorld (currPort, currDev);						// restore the original GWorld

		HUnlock ((*theControl) -> contrlData);

		// note that we redraw it even if the actual control value has not changed as a result of the drag -
		// this ensures that the indicator snaps into a position representing an exact value.
		(*theControl) -> contrlValue = newValue;
		DrawTheControl (theControl);
	}
}