/*
	File Name:	RecordToDisk XCMD.c
*/

/*************************************************************************

	RecordToDisk

	by Joe Zobkiw
	
	Records to a disk-based AIFF file in HyperCard.

	Form: RecordToDisk [horizontal_loc_of_record_dlog, vertical_loc_of_record_dlog]
	
*************************************************************************/

#include <Sound.h>
#include <SoundInput.h>
#include <Script.h>
#include <HyperXCmd.h>
#include <string.h>
#include "A4Stuff.h"
#include "SetupA4.h"

/*************************************************************************

	#defines

*************************************************************************/
   
#define kMinParamCount	0
#define kMaxParamCount	2
#define kErrorFlag		(short)-1

/*************************************************************************

	function prototypes

*************************************************************************/
   
pascal 	void main(XCmdPtr xp);
Handle	ConcatErrorStr(XCmdPtr xp, char *ch, OSErr err);
void	SetError(XCmdPtr xp, char *ch, OSErr err);
OSErr	GetLocOfCardWindow(XCmdPtr xp, Point *pt);
Handle	CopyStrToHand(char *ch);
long	HandleToNum(XCmdPtr xp, Handle h);
char*	ToCStr(char *ch);
char*	ToPStr(char *ch);

/*************************************************************************

	main

*************************************************************************/

pascal void main(XCmdPtr xp)
{
	StandardFileReply	reply;
	Point				corner;
	OSErr				err = noErr;
	short				refNum = -1;
	Boolean				cancelled = false;
	Boolean				fileCreated = false;
	Str32				tempStr;
	
#ifndef powerc
	long	oldA4 = SetCurrentA4();
	RememberA4();
#endif

	// check parameter count
	if ((xp->paramCount != kMinParamCount) && (xp->paramCount != kMaxParamCount)) {
		SetError(xp, "Error: Form = RecordToDisk [h, v]. ", 0);
		goto fail;
	}
	
	if (xp->paramCount == 2) {
		// use location as passed by scripter
		corner.h = (short)HandleToNum(xp, xp->params[0]);
		corner.v = (short)HandleToNum(xp, xp->params[1]);
	} else {
		// try to get the card window location to align dialog in relation to it
		err = GetLocOfCardWindow(xp, &corner);
		if (err == kErrorFlag) {
			SetError(xp, "Error: Couldnt get location of card window. ", 0);
			goto fail;
		}
	}
	
	// first ask the user to save the file
	StandardPutFile("\pSave AIFF file as:", "\pAIFF Audio", &reply);
	if (!reply.sfGood) {
		SetError(xp, "cancel", 0);
		goto exit;
	}
	
	// attempt to update the screen
	ZeroToPas(xp, "Go to this card", tempStr);
	SendHCMessage(xp, tempStr);

	// if we are replacing a file, do so
	if (reply.sfReplacing) {
		err = FSpDelete(&reply.sfFile);
		if (err != noErr) {
			SetError(xp, "Error: Couldnt replace file. ", err);
			goto fail;
		}
	}

	// create the new file
	err = FSpCreate(&reply.sfFile, '????', 'AIFF', smRoman);
	if (err != noErr) {
		SetError(xp, "Error: Couldnt create sound file. ", err);
		goto fail;
	}
	fileCreated = true;
	
	// open the file
	err = FSpOpenDF(&reply.sfFile, fsRdWrPerm, &refNum);
	if ((err != noErr) || (refNum == -1)) {
		SetError(xp, "Error: Couldnt open sound file. ", err);
		goto fail;
	}
		
	// record to the file, first tell HyperCard we are about to do
	// something with sound.
	BeginXSound(xp, nil);
	err = SndRecordToFile(nil, corner, siBestQuality, refNum);
	EndXSound(xp);
	cancelled = (err == userCanceledErr);
	if (cancelled) {
		SetError(xp, "cancel", 0);
		goto fail;
	}
	if (err != noErr) {
		SetError(xp, "Error: Couldnt record to sound file. ", err);
		goto fail;
	}
	
	goto exit;
fail:
	// clean up
	if (refNum != -1)
		FSClose(refNum);
	refNum = -1;
	if (fileCreated)
		FSpDelete(&reply.sfFile);
exit:
	
	if (refNum != -1) {
		// close the file
		FSClose(refNum);
		// flush the volume
		FlushVol(nil, reply.sfFile.vRefNum);
		if (cancelled)
			FSpDelete(&reply.sfFile);
	}
	
#ifndef powerc
	SetA4(oldA4);
#endif

	return;
}

/*************************************************************************

	

*************************************************************************/

Handle	ConcatErrorStr(XCmdPtr xp, char *ch, OSErr err)
{
	Str255	str1;
	Str32	str2;
	
	if (err == noErr) {
		return((Handle)CopyStrToHand(ch));
	} else {
		strcpy((char*)str1, ch);
		NumToString((long)err, str2);
		ToCStr((char*)str2);
		strcat((char*)str1, (char*)str2);
		return((Handle)CopyStrToHand((char*)str1));
	}
}

/*************************************************************************

	

*************************************************************************/

void	SetError(XCmdPtr xp, char *ch, OSErr err)
{
	xp->returnValue = (Handle)ConcatErrorStr(xp, ch, err);
}

/*************************************************************************

	

*************************************************************************/

OSErr	GetLocOfCardWindow(XCmdPtr xp, Point *pt)
{
	Handle	h;
	char	str[256];
	
	strcpy(str, "item 1 of rect of card window");
	h = EvalExpr(xp, (StringPtr)ToPStr(str));
	if (xp->result == noErr) {
		pt->h = HandleToNum(xp, h);
		DisposeHandle(h);
		
		strcpy(str, "item 2 of rect of card window");
		h = EvalExpr(xp, (StringPtr)ToPStr(str));
		if (xp->result == noErr) {
			pt->v = HandleToNum(xp, h);
			DisposeHandle(h);
			return noErr;
		}
	}
	return(kErrorFlag);
}

/*************************************************************************

	

*************************************************************************/

Handle	CopyStrToHand(char *ch)
{
	Handle	h;
	
	h = NewHandleClear((long)strlen(ch) + 1);
	if (h)
		strcpy((char*)*h, ch);
	return h;
}

/*************************************************************************

	

*************************************************************************/

long	HandleToNum(XCmdPtr xp, Handle h)
{
	char	str[32];
	long	num;
	
	strcpy(str, *h);
	num = StrToLong(xp, (StringPtr)ToPStr(str));
	return num;
}

/*************************************************************************

	

*************************************************************************/

char*	ToCStr(char *ch)
{
	unsigned char len, i;
	
	len = ch[0];
	for (i=0;i<len;++i)
		ch[i] = ch[i+1];
	ch[len] = 0;
	return ch;
}

/*************************************************************************

	

*************************************************************************/

char*	ToPStr(char *ch)
{
	unsigned char len, i;
	
	for (i=0,len=0;ch[i]!=0;++i)
		++len;
	while (i--)
		ch[i+1] = ch[i];
	ch[0] = len;
	return ch;
}
