/*
	File Name:		ColorFill.c
*/

/*******************************************************************************

	Fade
	
*******************************************************************************/

#include "Photoshop.h"

#ifdef __powerc
enum {
	uppPhotoshopFilterProcInfo = kPascalStackBased
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(Ptr)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(Ptr)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(Ptr)))
};
ProcInfoType __procinfo = uppPhotoshopFilterProcInfo;
#endif

/* typedefs */

#define itemForegroundColor	3
#define itemBackgroundColor	4
#define itemShowTimer		6

typedef struct TParameters {
	short 			whichColor;
	Boolean			fShowTimer;
	unsigned long	startTicks;
	unsigned long	endTicks;
} TParameters, *PParameters, **HParameters;

typedef struct Globals {
	short 			result;
	FilterRecord 	*stuff;
} Globals, *GPtr, **GHdl;

#define gResult ((*globals).result)
#define gStuff  ((*globals).stuff)

/* function prototypes */

void InitGlobals(GPtr globals);
void DoAbout(GPtr globals);
void DoParameters(GPtr globals);
void DoPrepare(GPtr globals);
void DoStart(GPtr globals);
void DoContinue(GPtr globals);
void DoFinish(GPtr globals);

void DoFilterRect(GPtr globals);

/*******************************************************************************

	main
	
*******************************************************************************/

pascal void main (short selector, FilterRecord *stuff, long *data, short *result)
{
	Globals 	globalValues;
	GPtr 		globals = &globalValues;
	
	if (!*data) {
		InitGlobals(globals);
		*data = (long)NewHandle(sizeof(Globals));
		if (!*data) {
			*result = memFullErr;
			return;
		}
		**(GHdl)*data = globalValues;
	}

	globalValues = **(GHdl)*data;

	gStuff = stuff;
	gResult = noErr;

	switch(selector) {

		case filterSelectorAbout:
			DoAbout(globals);
			break;

		case filterSelectorParameters:
			DoParameters(globals);
			break;
			
		case filterSelectorPrepare:
			DoPrepare(globals);
			break;
		
		case filterSelectorStart:
			DoStart(globals);
			break;
		
		case filterSelectorContinue:
			DoContinue(globals);
			break;
		
		case filterSelectorFinish:
			DoFinish(globals);
			break;
			
		default:
			gResult = filterBadParameters;
		
	}

	*result = gResult;
	**(GHdl)*data = globalValues;
}

/*******************************************************************************

	InitGlobals
	
	Initialize any globals that may need it.
	
*******************************************************************************/

void InitGlobals(GPtr globals)
{

}

/*******************************************************************************

	DoAbout
		
*******************************************************************************/

void DoAbout(GPtr globals)
{
	ShowAbout(16000);
}

/*******************************************************************************

	DoParameters
	
	Asks the user for the plug-in filter module's parameters. Note that
  	the image size information is not yet defined at this point. Also, do
   	not assume that the calling program will call this routine every time the
   	filter is run (it may save the data held by the parameters handle) and pass
   	them in through this call.
   	
*******************************************************************************/

#define kOptionsDialogID 16001

void DoParameters(GPtr globals)
{
	short 		item, whichColor;
	DialogPtr 	dp;
	DialogTHndl dt;
	Boolean		done = false, fShowTimer;
	
	// if our parameters have not been saved and passed in by the host program
	// we can allocate new ones and initialize them.
	if (!gStuff->parameters) {
		gStuff->parameters = NewHandle((long)sizeof(TParameters));
		if (!gStuff->parameters) {
			gResult = memFullErr;
			return;
		}
		((PParameters)*gStuff->parameters)->whichColor = itemForegroundColor;
		((PParameters)*gStuff->parameters)->fShowTimer = false;
	}
	
	// load our DLOG resource in order to allow Photoshop to prepare for movable modalness
	dt = (DialogTHndl)GetResource('DLOG', kOptionsDialogID);
	HNoPurge((Handle)dt);
	CenterDialog(dt);
	SetUpMoveableModal(dt, gStuff->hostSig);
	
	// load our dialog
	dp = GetNewDialog(kOptionsDialogID, nil, (WindowPtr) -1);
	
	// prepare the dialog
	SetDialogDefaultItem(dp, ok);
	SetDialogCancelItem(dp, cancel);
	SetDialogTracksCursor(dp, true);
	SetRadioGroupState(dp, itemForegroundColor, itemBackgroundColor, ((PParameters)*gStuff->parameters)->whichColor);
	SetCheckBoxState(dp, itemShowTimer, ((PParameters)*gStuff->parameters)->fShowTimer);
	SetArrowCursor();
	
	// be movable and modal and perform the standard dialog item handling
	// since we don't have any special items that need special handling
	while (!done) {
		MoveableModalDialog(dp, gStuff->processEvent, nil, &item);
		switch (item) {
			case ok:
				whichColor = GetRadioGroupState(dp, itemForegroundColor, itemBackgroundColor);
				((PParameters)*gStuff->parameters)->whichColor = whichColor;
				fShowTimer = GetCheckBoxState(dp, itemShowTimer);
				((PParameters)*gStuff->parameters)->fShowTimer = fShowTimer;
				done = true;
				break;
				
			case cancel:
				done = true;
				gResult = 1;
				break;
				
			case itemForegroundColor:
			case itemBackgroundColor:
			case itemShowTimer:
				PerformStandardDialogItemHandling(dp, item);
				break;
				
			default:
				break;
		}
	}
		
	// nuke the dialog
	DisposDialog(dp);
	HPurge((Handle) dt);
}

/*******************************************************************************

	DoPrepare
	
   	Prepare to filter an image.	If the plug-in filter needs a large amount
   	of buffer memory, this routine should set the bufferSpace field to the
   	number of bytes required. You can also set the bufferSpace field to 0 and
   	use the buffer and handle suites to allocate memory later.
   	
*******************************************************************************/

void DoPrepare(GPtr globals)
{
	gStuff->bufferSpace = 0;
}

/*******************************************************************************

	DoStart
   	
*******************************************************************************/

void DoStart(GPtr globals)
{
	int16 	row;
	int32 	totalLines = gStuff->filterRect.bottom - gStuff->filterRect.top;
	
	// insure that the advance state is available
	if (!WarnAdvanceStateAvailable()) {
		gResult = 1;
		goto done;
	}
	
	// make sure things are still a go
	if (gResult != noErr)
		goto done;
	
	// request first & last planes to process next
	gStuff->inLoPlane = gStuff->outLoPlane = 0;
	gStuff->inHiPlane = gStuff->outHiPlane = gStuff->planes - 1;
	
	// request area of image to work on, first fill in left/right bounds
	gStuff->inRect.left = gStuff->outRect.left = gStuff->filterRect.left;
	gStuff->inRect.right = gStuff->outRect.right = gStuff->filterRect.right;
	
	// start the timer
	if (((PParameters)*gStuff->parameters)->fShowTimer)
		((PParameters)*gStuff->parameters)->startTicks = TickCount();
	
	// for each row of the image, filter it
	for (row = gStuff->filterRect.top; row < gStuff->filterRect.bottom; ++row) {
		
		// update the progress if needed
		UpdateProgress(row - gStuff->filterRect.top, totalLines);
	
		// check for a cancellation by the user
		if (TestAbort()) {
			gResult = userCanceledErr;
			goto done;
		}
		
		// set the rectangle to point to the row we are about to work on
		gStuff->inRect.top = gStuff->outRect.top = row;
		gStuff->inRect.bottom = gStuff->outRect.bottom = row + 1;
		
		// call the advance state procedure
		gResult = AdvanceState();
		
		// check for errors
		if (gResult != noErr)
			goto done;
		
		// filter the row
		DoFilterRect(globals);
	}

done:
	// finish timing
	if (((PParameters)*gStuff->parameters)->fShowTimer)
		((PParameters)*gStuff->parameters)->endTicks = TickCount();

	// set these rects to be 0 in size since we are done
	SetRect(&gStuff->inRect, 0, 0, 0, 0);
	SetRect(&gStuff->outRect, 0, 0, 0, 0);
}

/*******************************************************************************

	DoContinue
   	
   	This function is called repeatedly while inRect or outRect are non-empty.
   	Since we do all of our work in the DoStart function, we never get here.
*******************************************************************************/

void DoContinue(GPtr globals)
{
	SetRect(&gStuff->inRect, 0, 0, 0, 0);
	SetRect(&gStuff->outRect, 0, 0, 0, 0);
}

/*******************************************************************************

	DoFinish
   	
   	Called only if DoStart returns noErr. This allows the filter to clean up.
   	This routine is also called if the user cancels.
   	
*******************************************************************************/

void DoFinish(GPtr globals)
{
	// display timer results if requested
	if (((PParameters)*gStuff->parameters)->fShowTimer) {
		unsigned long	totalTicks;
		Str32			totalTicksStr;
		
		totalTicks = ((PParameters)*gStuff->parameters)->endTicks -
						((PParameters)*gStuff->parameters)->startTicks;
		NumToString(totalTicks, totalTicksStr);
		ParamText(totalTicksStr, "\p", "\p", "\p");
		InitCursor();
		NoteAlert(16500, nil);
		ParamText("\p", "\p", "\p", "\p");
	}
}

/*******************************************************************************

	DoFilterRect
   	
   	This function is called for each section of an image that we need to edit.
   	In our case, we pass this function a row of the image at a time.
*******************************************************************************/

void DoFilterRect(GPtr globals)
{
	register	short 		width = gStuff->filterRect.right - gStuff->filterRect.left;
	register	short 		whichColor = ((PParameters)*gStuff->parameters)->whichColor;
	register 	unsigned8 	*srcPtr = (unsigned8 *)gStuff->inData;
	register	unsigned8 	*dstPtr = (unsigned8 *)gStuff->outData;
	register 	short		plane;
	
	// for each pixel in the row, do something to it
	while (--width >= 0) {
		
		// for each plane above 4, do nothing to each pixel
		for (plane = gStuff->planes - 1; plane >= 4; --plane)
			;
		
		// for the rest of the planes, edit the pixel value
		for (; plane >= 0; --plane)
			dstPtr[plane] = (whichColor == itemForegroundColor) ? gStuff->foreColor[plane] : gStuff->backColor[plane];
		
		// increment to the next pixel
		srcPtr += gStuff->inHiPlane - gStuff->inLoPlane + 1;
		dstPtr += gStuff->planes; // one unsigned8 per plane
	}
}