/*
	File Name:		TVTube.c
*/

// *****************************************************************************
//
//	TVTube.c
//
// 
//	C source file for TV Tube Effect filter plug-in for Photoshop.
// 
//	Original code from Dissolve.c:
//		Copyright 1990 by Thomas Knoll.
//		Copyright 1991-95 by Adobe Systems, Inc. All rights reserved.
//		Converted to work with Metrowerks CodeWarrior by Joe Zobkiw. (3/2/95)
//	Copyright 1995 by Troy Gaul.
// 
//	CHANGE HISTORY: (most recent first)
//		  3/5/95 - TG - Moved parameters dialog into GetParameters function.
//						Now calls GetParameters from DoContinue instead of
//						DoParameters (to facilitate a possible preview area).
//		  3/4/95 - TG - Started from Dissolve sample.
// *****************************************************************************

#include "Photoshop.h"

// 
//
//	Constants
//
// 
enum {
	kAboutBoxDialogID = 16000,
	kSettingsDialogID = 16001,
	
	kCommentStringListID = 16000
};

enum {
	kOddFieldsItem = 4,
	kEvenFieldsItem
};


// 
//
//	__procinfo
//
// 
#ifdef __powerc

enum {
	uppPhotoshopFilterProcInfo = kPascalStackBased
		 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short)))
		 | STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(Ptr)))
		 | STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(Ptr)))
		 | STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(Ptr)))
};

ProcInfoType __procinfo = uppPhotoshopFilterProcInfo;


#endif
// 
//
//	TParameters
//
// 
typedef struct TParameters {
	short oddDelta;
	short evenDelta;
} TParameters, *PParameters, **HParameters;


// 
//
//	Globals
//
// 
typedef struct Globals {
	short result;
	FilterRecord *stuff;
	
	short wantsParamaters;
	short inParamaterStage;
	
	short row;
} Globals, *GPtr, **GHdl;


// Accessor macros.
#define gResult 			(globals->result)
#define gStuff  			(globals->stuff)

#define gWantsParamaters	(globals->wantsParamaters)
#define gInParamaterStage	(globals->inParamaterStage)

#define gRow				(globals->row)


// 
//
//	Prototypes
//
// 
void DoAbout(GPtr globals);
void DoParameters(GPtr globals);
void DoPrepare(GPtr globals);
void DoStart(GPtr globals);
void DoContinue(GPtr globals);
void DoFinish(GPtr globals);
void InitGlobals(GPtr globals);


// 
//
//	Utility macros
//
// 
#define ClipInRange(x, min, max) \
	(((x) < (min)) ? (min) : (((x) > (max)) ? (max) : (x)))


// 
//
//	main
//
// 
//		All calls to the plug-in module come through this routine. It must be
//		placed first in the resource. To achieve this, most development systems
//		require that this be the first routine in the source.
// 
pascal void 
main(short selector, FilterRecord *stuff, long *data, short *result)
{
	Globals globalValues;
	GPtr globals = &globalValues;
	
	if (!*data) {
		InitGlobals(globals);

		*data = (long) NewHandle(sizeof(Globals));
		
		if (!*data) {
			*result = memFullErr;
			return;
		}
			
		**((GHdl) *data) = globalValues;
	}
		
	globalValues = **((GHdl) *data);
		
	gStuff = stuff;
	gResult = noErr;
		
	switch (selector) {
		
		case filterSelectorAbout:
			DoAbout(globals);
			break;
			
		case filterSelectorParameters:
			DoParameters(globals);
			break;
			
		case filterSelectorPrepare:
			DoPrepare(globals);
			break;
		
		case filterSelectorStart:
			DoStart(globals);
			break;
		
		case filterSelectorContinue:
			DoContinue(globals);
			break;
		
		case filterSelectorFinish:
			DoFinish(globals);
			break;
			
		default:
			gResult = filterBadParameters;
			break;
			
	}
		
	*result = gResult;
	
	**((GHdl) *data) = globalValues;
}


// 
//
//	InitGlobals
//
// 
void 
InitGlobals(GPtr globals)
{
	gWantsParamaters = false;
	gInParamaterStage = false;

	gRow = 0;
}


// 
//
//	SetupNextFilterStrip
//
// 
//		Set the inRect and outRect for the image strip indicated by gRow.
// 
static void
SetupNextFilterStrip(GPtr globals)
{
	gStuff->inRect.top    = gStuff->outRect.top    = gRow;
	gStuff->inRect.bottom = gStuff->outRect.bottom = gRow + 1;
}


// 
//
//	SetupFirstFilterStrip
//
// 
static void
SetupFirstFilterStrip(GPtr globals)
{
	// Get all the planes.
	gStuff->inLoPlane = gStuff->outLoPlane = 0;
	gStuff->inHiPlane = gStuff->outHiPlane = gStuff->planes - 1;
	
	// Get the entire width of the filter rect each time.
	gStuff->inRect.left  = gStuff->outRect.left  = gStuff->filterRect.left;
	gStuff->inRect.right = gStuff->outRect.right = gStuff->filterRect.right;
	
	// Start with the first scanline.
	gRow = gStuff->filterRect.top;
	SetupNextFilterStrip(globals);
}


// 
//
//	GetParameters
//
// 
//		Display the parameters dialog for the user to enter values.  Returns
//		true when it is done.
// 
static Boolean 
GetParameters(GPtr globals)
{
	long odd, even;
	short item;
	DialogPtr dp;
	DialogTHndl dt;
	PParameters params = (PParameters) *gStuff->parameters;

	// Prepare the DLOG resource for use.
	dt = (DialogTHndl) GetResource('DLOG', kSettingsDialogID);
	if (!dt) {
		gResult = ResError() ? ResError() : resNotFound;
		return true;
	}
	
	HNoPurge((Handle) dt);
	
	CenterDialog(dt);
	SetUpMoveableModal(dt, gStuff->hostSig);
	
	// Create the dialog.
	dp = GetNewDialog(kSettingsDialogID, nil, (WindowPtr) -1);
	if (!dp) {
		gResult = memFullErr;
		return true;
	}

	// Set the dialog up.
	(void) SetDialogDefaultItem(dp, ok);
	(void) SetDialogCancelItem(dp, cancel);
	(void) SetDialogTracksCursor(dp, true);
	
	StuffNumber(dp, kOddFieldsItem,  params->oddDelta);
	StuffNumber(dp, kEvenFieldsItem, params->evenDelta);
	
	SetArrowCursor();

	SelectTextItem(dp, kOddFieldsItem); 

	// Loop to handle the dialog items.
	do {
		MoveableModalDialog(dp, gStuff->processEvent, nil, &item);

		if (item == ok) {
			// Validate the input.
			if (!FetchNumber(dp, kOddFieldsItem, -255, 255, &odd))
				item = 0;
			else if (!FetchNumber(dp, kEvenFieldsItem, -255, 255, &even))
				item = 0;
		}

	} while (item != ok && item != cancel);

	DisposeDialog(dp);
	HPurge((Handle) dt);

	// Handle the dismissal of the dialog.
	if (item == ok) {
		params->oddDelta  = odd;
		params->evenDelta = even;
	
	} else {
		gResult = userCanceledErr;
	}

	return true;
		// Return true because we dont want to be called again.
}


// 
//
//	FilterRect
//
// 
//		Filter part of the area.
// 
static void 
FilterRect(GPtr globals)
{
	Byte *srcPtr = (unsigned8*) gStuff->inData;
	Byte *dstPtr = (unsigned8*) gStuff->outData;
	
	Boolean isOdd = (gStuff->inRect.top & 0x0001);
	short delta = isOdd ? ((PParameters) *gStuff->parameters)->oddDelta 
						: ((PParameters) *gStuff->parameters)->evenDelta;

	short srcColBytes = gStuff->inHiPlane - gStuff->inLoPlane + 1;
	short dstColBytes = gStuff->planes;
	short count = gStuff->filterRect.right - gStuff->filterRect.left;
	
	while (count--) {
			short plane;

#if 1
			// All planes.
			for (plane = gStuff->planes - 1; plane >= 0; --plane) {
				short value = srcPtr[plane] + delta;
				dstPtr[plane] = ClipInRange(value, 0, 255);
			}
#elif 0
			// Only some planes.			
			for (plane = gStuff->planes - 1; plane >= 4; --plane)
		//		dstPtr[plane] = 255;
				;
				
			for (; plane >= 0; --plane) {
				short value = srcPtr[plane] + delta;
				dstPtr[plane] = ClipInRange(value, 0, 255);
			}
#endif
		
		srcPtr += srcColBytes;
		dstPtr += dstColBytes;
	}
}

// 
//
//	DoAbout
//
// 
//		Displays the about dialog box for the plug-in module.
// 
void DoAbout(GPtr globals)
{
	ShowAbout(kAboutBoxDialogID);
}


// 
//
//	DoParameters
//
// 
//		Asks the user for the plug-in filter module's parameters. Note that
//		the image size information is not yet defined at this point. Also, do
//		not assume that the calling program will call this routine every time 
//		the filter is run (it may save the data held by the parameters handle 
//		in a macro file).
// 
void 
DoParameters(GPtr globals)
{
	// Create a parameter block and fill it out with defaults.
	if (!gStuff->parameters) {
		gStuff->parameters = NewHandle(sizeof(TParameters));

		if (!gStuff->parameters) {
			gResult = memFullErr;
			return;
		}

		{
			PParameters params = (PParameters) *gStuff->parameters;
		
			params->oddDelta  =  50;
			params->evenDelta = -50;
		}
		
	// Make sure the incoming parameters are valid.
	} else {
		PParameters params = (PParameters) *gStuff->parameters;
		
		params->oddDelta  = ClipInRange(params->oddDelta,  -255, 255);
		params->evenDelta = ClipInRange(params->evenDelta, -255, 255);
			// Just in case the param block got messed up (it may have been
			// stored somewhere by the host application).
	}

	gWantsParamaters = true;
		// Set the flag to indicate that a call to DoParameters has been
		// received.
}


// 
//
//	DoPrepare
//
// 
//		Prepare to filter an image.	If the plug-in filter needs a large amount
//		of buffer memory, this routine should set the bufferSpace field to the
//		number of bytes required.
// 
void 
DoPrepare(GPtr globals)
{
	gStuff->bufferSpace = 0;
}


// 
//
//	DoStart
//
// 
//		Sets up filtering and grabs the first scan line of the filterRect for
//		the first call to DoContinue.
// 
void 
DoStart(GPtr globals)
{
	SetupFirstFilterStrip(globals);

	// Inform DoContinue that we are in the GetParamater stage if we need to.
	if (gWantsParamaters) {
		gInParamaterStage = true;
		gWantsParamaters = false;
	}
}


// 
//
//	DoContinue
//
// 
//		In order for a plug-in to be able to work with versions of Photoshop
//		earlier than 3.0 (and other host applications as well), we need to
//		actually implement the filtering in DoContinue.  Otherwise we could
//		just use the nifty new AdvanceState callback routine and do all the
//		filtering in DoStart.
// 
void 
DoContinue(GPtr globals)
{
	if (gInParamaterStage) {
		Boolean done = GetParameters(globals);
			// By bring up the parameters dialog here, we can add code to it
			// that would allow it to support a preview area by having it
			// request parts of the image and get called back.  This mechanism
			// is even simpler with Photoshop 3.0 and later by using the
			// AdvanceState callback function.

		if (done) {
			gInParamaterStage = false;
			SetupFirstFilterStrip(globals);
		}
			
	} else {
		// Show the current progress state.
		int32 total = gStuff->filterRect.bottom - gStuff->filterRect.top;
		UpdateProgress(gRow - gStuff->filterRect.top, total);
		
		// Check for command-period or cancel.
		if (TestAbort()) {
			gResult = userCanceledErr;

		} else {
			// Filter the current strip.
			FilterRect(globals);
	
			// Setup for next call to DoContinue.
			gRow++;
			if (gRow < gStuff->filterRect.bottom) {
				SetupNextFilterStrip(globals);
	
			// Were done, so signal this fact to the host.
			} else {
				SetRect(&gStuff->inRect,  0, 0, 0, 0);
				SetRect(&gStuff->outRect, 0, 0, 0, 0);
			}
		}
	}
}


// 
//
//	DoFinish
//
// 
//		Do any necessary clean-up.
// 
void 
DoFinish(GPtr globals)
{
}


// 
