/*
	File Name:		ComponentTester.c
*/

/*
	Assumes Component Manager 3.0 or later.
*/

#include <GestaltEqu.h>
#include <Components.h>
#include <Icons.h>

#include "FatComponent.h"
#include "FatComponentPrivate.h"
#include "ComponentTester.h"

FatComponent		gFatComponent = nil;
Handle				gIconSuite = nil;

void main(void)
{
	unsigned long 	randSeed;
	DialogPtr		d = nil;
	Boolean			fDone = false;
	OSErr			err = noErr;
	GrafPtr			savePort;
	UserItemUPP		drawComponentIconUPP;
	
	// initialize the Mac
	InitGraf(&qd.thePort);
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	GetDateTime((unsigned long *)&randSeed);
	LMSetRndSeed(randSeed);
	DrawMenuBar();

	// get our main dialog box
	d = GetNewDialog(dlogMainID, nil, (WindowPtr)-1);
	if (d == nil) ExitToShell();
	
	// set it up properly
	GetPort(&savePort);
	SetPort(d);
	SetWindowFont(d, geneva, 9, 0, srcCopy);
	drawComponentIconUPP = NewUserItemProc(DrawComponentIcon);
	SetUserItemProc(d, itemComponentIcon, (ProcPtr)drawComponentIconUPP);
	DisableControl(d, itemUnregister);
	DisableControl(d, itemBeep);
	DisableControl(d, itemFlash);
	SelIText(d, itemBeepTimes, 0, 32767);
	ShowWindow(d);
	
	// loop until done
	while (!fDone) {
		short	itemHit;
		ModalDialog(nil, &itemHit);
		switch(itemHit) {
		
			// Component Manager will automatically unregister 
			// a component here, if one is registered.
			case itemQuit:
				fDone = true;
				break;
			
			// allow the user to choose a component file to register
			case itemRegister:
				err = DoRegister();
				if (err == noErr) {
					DisableControl(d, itemRegister);
					EnableControl(d, itemUnregister);
					EnableControl(d, itemBeep);
					EnableControl(d, itemFlash);
					
					// get the icon suite of the component in order to draw it
					// in our dialog
					err = GetComponentIconSuite((Component)gFatComponent, &gIconSuite);
					if (err != noErr) {
						ShowError(err, "\pgetting the components icon suite");
						err = noErr;
					}
				} else {
					ShowError(err, "\pregistering the component");
					err = noErr;
				}
				break;
				
			// unregister any registered component
			case itemUnregister:
				err = DoUnregister();
				if (err == noErr) {
					EnableControl(d, itemRegister);
					DisableControl(d, itemUnregister);
					DisableControl(d, itemBeep);
					DisableControl(d, itemFlash);
					
					// dispose of the icon suite if any
					if (gIconSuite) {
						err = DisposeIconSuite(gIconSuite, true);
						if (err != noErr) {
							ShowError(err, "\pdisposing the components icon suite");
							err = noErr;
						} else gIconSuite = nil;
					}
					CallUserItemProc(drawComponentIconUPP, d, itemComponentIcon);	// update icon
				} else {
					ShowError(err, "\punregistering the component");
					err = noErr;
				}
				break;
				
			// call the current component to beep x number of times
			case itemBeep:
				{
					Str255	strBeepTimes;
					long	beepTimes;
					
					GetItemText(d, itemBeepTimes, strBeepTimes);
					StringToNum(strBeepTimes, &beepTimes);
					
					err = Beep((short)beepTimes);
					if (err != noErr) {
						ShowError(err, "\pattempting to beep");
						err = noErr;
					}
				}
				break;
				
			// call the current component to flash the menubar x number of times
			case itemFlash:
				{
					Str255	tempStr;
					long	flashTimes, flashDelay;
					
					GetItemText(d, itemFlashTimes, tempStr);
					StringToNum(tempStr, &flashTimes);
					GetItemText(d, itemFlashDelay, tempStr);
					StringToNum(tempStr, &flashDelay);
					
					err = Flash((short)flashTimes, flashDelay);
					if (err != noErr) {
						ShowError(err, "\pattempting to flash");
						err = noErr;
					}
				}
				break;
				
			default:
				break;
		}
	}
	
	// clean up
	DisposeDialog(d);
	DisposeRoutineDescriptor(drawComponentIconUPP);
	SetPort(savePort);
}

/*******************************************************************************

	DoRegister

	Allow the user to pick a file of type 'thng', open it, get the 'thng' resource
	of ID 128 and then call RegisterComponentResource on it. Once the component
	is registered we close the resource file and continue.
	
*******************************************************************************/

OSErr DoRegister(void)
{
	OSErr				err = noErr;
	SFTypeList			typeList = {'thng', '????', '????', '????'};
	StandardFileReply	reply;
	
	StandardGetFile(nil, 1, typeList, &reply);
	if (reply.sfGood) {
	
		short fRefNum = FSpOpenResFile(&reply.sfFile, fsRdPerm);
	
		if (fRefNum != -1) {
			ComponentResourceHandle thngResH = (ComponentResourceHandle)Get1Resource(kComponentResourceType, 128);
			
			if (thngResH) {
				gFatComponent = (FatComponent)RegisterComponentResource(thngResH, registerComponentNoDuplicates);
				ReleaseResource((Handle)thngResH);
				if (gFatComponent == nil)
					err = -1;
			} else err = ResError() ? ResError() : resNotFound;
			
			CloseResFile(fRefNum);
		
		} else err = ResError() ? ResError() : fnfErr;
	
	} else err = -128;
	return err;
}

/*******************************************************************************

	DoUnregister

	Simply call UnregisterComponent on the current registered component.
	
*******************************************************************************/

OSErr DoUnregister(void)
{
	OSErr	err = noErr;
	
	if (gFatComponent) {
		err = UnregisterComponent((Component)gFatComponent);
		if (err == noErr)
			gFatComponent = nil;
	}
	
	return err;
}

/*******************************************************************************

	Beep

	Call the current registered component to beep beepTimes.
	
*******************************************************************************/

OSErr Beep(short beepTimes)
{
	OSErr 				err = noErr;
	ComponentInstance	ci;
	
	ci = OpenComponent((Component)gFatComponent);
	if (ci) {
		short	outBeepTimes;
		
		err = DoBeep(ci, beepTimes, &outBeepTimes);
		if (err != noErr) ShowError(err, "\pattempting to DoBeep");
		
		if (outBeepTimes != beepTimes) ShowError(kGenericError, "\pcomparing beepTimes to outBeepTimes");
		
		err = CloseComponent(ci);
		if (err != noErr) ShowError(err, "\pattempting to CloseComponent");
	} else err = kGenericError;
	
	return err;
}

/*******************************************************************************

	Flash

	Call the current registered component to flash flashTimes.

*******************************************************************************/

OSErr Flash(short flashTimes, long flashDelay)
{
	OSErr 				err = noErr;
	ComponentInstance	ci;
	
	ci = OpenComponent((Component)gFatComponent);
	if (ci) {
		short	outFlashTimes;
		
		err = DoFlash(ci, flashTimes, flashDelay, &outFlashTimes);
		if (err != noErr) ShowError(err, "\pattempting to DoFlash");
		
		if (outFlashTimes != flashTimes) ShowError(kGenericError, "\pcomparing flashTimes to outFlashTimes");
		
		err = CloseComponent(ci);
		if (err != noErr) ShowError(err, "\pattempting to CloseComponent");
	} else err = kGenericError;
	
	return err;
}

/*******************************************************************************

	DrawComponentIcon

	User item proc to draw the components icon.
	
*******************************************************************************/

pascal void DrawComponentIcon(DialogPtr d, short theItem)
{
	Rect 	r;
	GrafPtr	savePort;
	
	GetPort(&savePort);
	SetPort(d);
	
	GetDialogItemRect(d, theItem, &r);
	if (gIconSuite) {
		OSErr err = PlotIconSuite(&r, atNone, ttNone, gIconSuite);
	} else {
		FillRect(&r, &qd.ltGray);
		FrameRect(&r);
	}
	
	SetPort(savePort);
}

/*******************************************************************************

	GetItemText

*******************************************************************************/

void GetItemText(DialogPtr dlg, short itemNumber, unsigned char* textStr)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, itemNumber, &iKind, &iHandle, &iRect);
	GetIText(iHandle, textStr);
}

/*******************************************************************************

	GetDialogItemRect

*******************************************************************************/

void GetDialogItemRect(DialogPtr dlg, short itemNumber, Rect *r)
{
	short	iKind;
	Handle	iHandle;

	GetDItem(dlg, itemNumber, &iKind, &iHandle, r);
}

/*******************************************************************************

	SetUserItemProc

*******************************************************************************/

void SetUserItemProc(DialogPtr dlg, short userItem, ProcPtr userProc)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;

	GetDItem(dlg, userItem, &iKind, &iHandle, &iRect);
	SetDItem(dlg, userItem, iKind, (Handle)userProc, &iRect);
}

/*******************************************************************************

	EnableControl

*******************************************************************************/

void EnableControl(DialogPtr dlg, short controlNumber)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, controlNumber, &iKind, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 0);
}

/*******************************************************************************

	DisableControl

*******************************************************************************/

void DisableControl(DialogPtr dlg, short controlNumber)
{
	short	iKind;
	Handle	iHandle;
	Rect	iRect;
	
	GetDItem(dlg, controlNumber, &iKind, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 255);
}

/************************************************************************* 
	
	SetWindowFont
	
**************************************************************************/

void SetWindowFont(DialogPtr d, short fontNum, short fontSize, Style fontStyle, short fontMode)
{
	FontInfo	fInfo;
	GrafPtr		savePort;
	
	GetPort(&savePort);
	SetPort(d);
	
	TextFont(fontNum);
    TextSize(fontSize);
    TextFace(fontStyle);
    TextMode(fontMode);
   	
   	GetFontInfo(&fInfo);

    (*((DialogPeek)d)->textH)->fontAscent = fInfo.ascent;
    (*((DialogPeek)d)->textH)->lineHeight = fInfo.ascent + fInfo.descent + fInfo.leading;
    (*((DialogPeek)d)->textH)->txFont = fontNum;
    (*((DialogPeek)d)->textH)->txFace = fontStyle;
    (*((DialogPeek)d)->textH)->txMode = fontMode;
    (*((DialogPeek)d)->textH)->txSize = fontSize;
    
    SetPort(savePort);
}

/*******************************************************************************

	ShowError

*******************************************************************************/

void ShowError(OSErr errorCode, unsigned char * errorStr)
{
	Str32	errorCodeStr;
	NumToString((long)errorCode, errorCodeStr);
	ParamText(errorCodeStr, errorStr, "\p", "\p");
	StopAlert(256, nil);
	ParamText("\p", "\p", "\p", "\p");
}