/*
	File Name:		FatComponentCommon.c
*/

#include "FatComponent.h"
#include "FatComponentCommon.h"
#include "FatComponentPrivate.h"
#include <OSUtils.h>

pascal ComponentResult FatOpen(ComponentInstance self)
{
	ComponentResult		result = noErr;
	PrivateGlobals**	globals;

	globals = (PrivateGlobals**)NewHandleClear(sizeof(PrivateGlobals));
	if (globals != nil) {
		
		// remember ourselves
		(*globals)->self = (Component)self;
		
		// tell the component manager that we have global storage
		SetComponentInstanceStorage(self, (Handle)globals);
	} else result = MemError() ? MemError() : memFullErr;	// NewHandleClear failed

	return result;
}

pascal ComponentResult FatClose(Handle storage, ComponentInstance self)
{
	ComponentResult		result = noErr;
	PrivateGlobals**	globals = (PrivateGlobals**) storage;

	if (globals != nil) {
		DisposeHandle((Handle)globals);
		globals = nil;
	}
	return result;
}

pascal ComponentResult FatCanDo(short selector)
{
	switch(selector) {
		
		// component Manager request codes
		case kComponentOpenSelect:
		case kComponentCloseSelect:
		case kComponentCanDoSelect:
		case kComponentVersionSelect:
		
		// our component request codes
		case kDoBeepSelect:
		case kDoFlashSelect:
			return true;
			break;
			
		case kComponentRegisterSelect:	// not supported
		default:						// unknown request
			return false;
			break;
	}
}

pascal ComponentResult FatVersion(void)
{
	return interfaceRevision;
}

pascal ComponentResult FatDoBeep(short inBeepTimes, short *outBeepTimes)
{
	ComponentResult	result = noErr;
	
	*outBeepTimes = inBeepTimes;
	if (inBeepTimes > 0) {
		short 	i;
		for (i=1;i<=inBeepTimes;++i)
			SysBeep(0);
	} else result = kGenericError;
	
	return result;
}

pascal ComponentResult FatDoFlash(short inFlashTimes, long inDelayTime, short *outFlashTimes)
{
	ComponentResult	result = noErr;
	
	*outFlashTimes = inFlashTimes;
	if (inFlashTimes > 0) {
		short 	i;
		long	outTicks;
		for (i=1;i<=inFlashTimes;++i) {
			FlashMenuBar(0);
			Delay(inDelayTime, &outTicks);
			FlashMenuBar(0);
			Delay(inDelayTime, &outTicks);
		}
	} else result = kGenericError;
	
	return result;
}
