/*
	File Name:		a5World.c
*/

#include "a5 World.h"

void main(void)
{
	unsigned long 	randSeed;
	A5Params		a5p;
	Rect			screenBitsBounds;
	
	// initialize the Mac
	InitGraf(&qd.thePort);
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	GetDateTime((unsigned long *)&randSeed);
	LMSetRndSeed(randSeed);
	DrawMenuBar();
	
	// create our own a5 world
	PrepareA5World(&a5p, &screenBitsBounds);
	
	// draw within it
	ForeColor(redColor);
	PaintRect(&screenBitsBounds);
	
	// restore the old a5 world
	RestoreA5World(&a5p);
	
	// wait for a mouse click
	while (!Button()) {}
	
	// update the screen
	UpdateRectangle(&screenBitsBounds);
	
	// flush any stray events
	FlushEvents(everyEvent, 0);
}

//
// Call this function before you want to draw when a5 may be invalid
//

void PrepareA5World(A5ParamsPtr pp, Rect *screenBitsBounds)
{
	//
	// save the zone and set to the System zone
	// this may not be needed but we used this code
	// in a system extension that used the system heap
	//
	
	pp->oldZone = GetZone();
	SetZone(SystemZone());
	
	//
	// save DeskHook and DragHook, which are low memory proc ptrs
	// then set them to 0 so they are ignored
	//
	
	pp->saveDeskHook = LMGetDeskHook();
	pp->saveDragHook = LMGetDragHook();
	LMSetDeskHook((UniversalProcPtr)0L);
	LMSetDragHook((UniversalProcPtr)0L);
	
	//
	// set ResLoad to true so our resources are loaded
	// may not be needed in your code
	//
	
	pp->savedResLoad = LMGetResLoad();
	SetResLoad(true);
	
	//
	// save the values of the Font Manager and QuickDraw init flags
	//
	
	pp->pFMExist = (Ptr)0x0D42;
	pp->pQDExist = (Ptr)0x08F3;
	pp->savedFMExist = *(pp->pFMExist);
	pp->savedQDExist = *(pp->pQDExist);
	
	//
	// set our a5 world, initialize it, and init the Font Manager
	//
	
	pp->oldA5 = SetA5((long)&(pp->qdWorld.thePort));
	InitGraf(&(pp->qdWorld.qd.thePort));
	if ((pp->savedFMExist == 0x00) && (LMGetWidthTabHandle() == nil))
		*(pp->pFMExist) = 0xFF;
	InitFonts();
	
	//
	// open a color port, requires Color QuickDraw
	//
	
	OpenCPort((CGrafPtr)(&pp->qdWorld.port));
	
	//
	// return the bounds of the a5 world we have created
	//
	
	*screenBitsBounds = pp->qdWorld.qd.screenBits.bounds;
}

//
// Call this function when you are through drawing into your a5 world
//

void RestoreA5World(A5ParamsPtr pp)
{
	//
	// close the color port
	//
	
	CloseCPort((CGrafPtr)(&pp->qdWorld.port));

	//
	// restore the old a5 world
	//
	
	SetA5(pp->oldA5);
	
	//
	// restore other saved settings
	//
	
	*(pp->pQDExist) = pp->savedQDExist;
	*(pp->pFMExist) = pp->savedFMExist;
	LMSetDeskHook(pp->saveDeskHook);
	LMSetDragHook(pp->saveDragHook);
	LMSetResLoad(pp->savedResLoad);
	SetZone(pp->oldZone);
}

//
// Call this function to update the entire screen
//

void UpdateRectangle(Rect *updateRect)
{
	RgnHandle	thisScreenBoundary = NewRgn();
	GrafPtr		oldPort;
	GrafPtr		theBigPicture;

	if (thisScreenBoundary) {
		RectRgn(thisScreenBoundary, updateRect);
		GetPort(&oldPort);
		GetWMgrPort(&theBigPicture);
		SetPort(theBigPicture);
		DrawMenuBar();
		PaintOne(nil, thisScreenBoundary);
		PaintBehind(LMGetWindowList(), thisScreenBoundary);
		SetPort(oldPort);
		DisposeRgn(thisScreenBoundary);
	}
}
